@@ -6,7 +6,7 @@
 import collections
 import gc
 import os
-from typing import Optional, Text, Union
+from typing import List, Optional, Text, Union
 
 from coremltools import (
     _LOWEST_ALLOWED_SPECIFICATION_VERSION_FOR_MILPROGRAM,
@@ -14,7 +14,8 @@ from coremltools import (
 )
 from coremltools import ComputeUnit as _ComputeUnit
 from coremltools import __version__ as _ct_version
-from coremltools._deps import _HAS_TF_1, _HAS_TF_2, _HAS_TORCH
+from coremltools import _logger as logger
+from coremltools._deps import _HAS_TF_1, _HAS_TF_2, _HAS_TORCH, _HAS_TORCH_EXPORT_API
 from coremltools.converters._profile_utils import _profile
 from coremltools.converters.mil._deployment_compatibility import (
     AvailableTarget,
@@ -23,8 +24,11 @@ from coremltools.converters.mil._deployment_compatibility import (
 from coremltools.converters.mil.converter import mil_convert
 from coremltools.converters.mil.input_types import (
     ClassifierConfig,
+    EnumeratedShapes,
     ImageType,
     InputType,
+    RangeDim,
+    Shape,
     TensorType,
 )
 from coremltools.converters.mil.mil import Program, types
@@ -32,7 +36,7 @@ from coremltools.converters.mil.mil.passes.defs.quantization import ComputePreci
 from coremltools.converters.mil.mil.passes.defs.quantization import FP16ComputePrecision
 from coremltools.converters.mil.mil.passes.graph_pass import PassOption as _PassOption
 from coremltools.converters.mil.mil.passes.pass_pipeline import PassPipeline
-from coremltools.models import _METADATA_SOURCE, _METADATA_VERSION
+from coremltools.models import _METADATA_SOURCE, _METADATA_SOURCE_DIALECT, _METADATA_VERSION
 from coremltools.models.utils import _MLPACKAGE_EXTENSION
 
 if _HAS_TF_1:
@@ -47,8 +51,13 @@ if _HAS_TF_2:
 if _HAS_TORCH:
     import torch
 
-    from coremltools.converters.mil.frontend.torch.load import \
-        _torchscript_from_model as pytorch_load
+    from coremltools.converters.mil.frontend.torch.load import (
+        _torchscript_from_spec as try_load_torchscript,
+    )
+
+    if _HAS_TORCH_EXPORT_API:
+        from torch.export import ExportedProgram
+
 
 
 @_profile
@@ -98,8 +107,12 @@ def convert(
 
         * PyTorch
 
-            - A `TorchScript <https://pytorch.org/docs/stable/jit.html>`_ object
-            - Path to a ``.pt`` file
+            - TorchScript Models:
+                - A `TorchScript <https://pytorch.org/docs/stable/jit.html>`_ object
+                - Path to a ``.pt`` file
+
+            - Torch Exported Models:
+                - A `ExportedProgram <https://pytorch.org/docs/stable/export.html#torch.export.ExportedProgram> ` object with `EDGE` dialect
 
     source : str (optional)
 
@@ -153,17 +166,27 @@ def convert(
               ``ct.utils.rename_feature`` API.
             - If ``dtype`` is not specified, it defaults to the ``dtype`` of the
               inputs in the TF model.
+            - For ``minimum_deployment_target >= ct.target.macOS13``, and with ``compute_precision`` in float 16 precision.
+              When ``inputs`` not provided or ``dtype`` not specified. The float 32 inputs defaults to float 16.
 
         * PyTorch:
-            - The ``inputs`` parameter is required.
-            - Number of elements in ``inputs`` must match the number of inputs
-              of the PyTorch model.
-            - ``inputs`` may be a nested list or tuple.
-            - ``TensorType`` and ``ImageType`` must have the ``shape`` specified.
-            - If the ``name`` argument is specified with ``TensorType`` or
-              ``ImageType``, the converted Core ML model will have inputs with
-              the same name.
-            - If ``dtype`` is missing, it defaults to float 32.
+
+            - TorchScript Models:
+                - The ``inputs`` parameter is required.
+                - Number of elements in ``inputs`` must match the number of inputs
+                of the PyTorch model.
+                - ``inputs`` may be a nested list or tuple.
+                - ``TensorType`` and ``ImageType`` must have the ``shape`` specified.
+                - If the ``name`` argument is specified with ``TensorType`` or
+                ``ImageType``, the converted Core ML model will have inputs with
+                the same name.
+                - If ``dtype`` is missing:
+                * For ``minimum_deployment_target <= ct.target.macOS12``, it defaults to float 32.
+                * For ``minimum_deployment_target >= ct.target.macOS13``, and with ``compute_precision`` in float 16 precision.
+                    It defaults to float 16.
+
+            - Torch Exported Models:
+                - The ``inputs`` parameter is not supported. ``inputs`` parameter is inferred from Torch ExportedProgram.
 
     outputs : list of ``TensorType`` or ``ImageType`` (optional)
 
@@ -203,13 +226,24 @@ def convert(
             - If specified, the ``name`` with ``TensorType`` or ``ImageType``
               must correspond to a node in the TF graph. In this case, the model
               will be converted up to that node.
+            - For ``minimum_deployment_target >= ct.target.macOS13``, and with ``compute_precision`` in float 16 precision.
+              If ``dtype`` not specified, the outputs inferred of type float 32
+              defaults to float 16.
 
         * PyTorch:
 
-            - If specified, the length of the list must match the number of
-              outputs returned by the PyTorch model.
-            - If ``name`` is specified, it is applied to the output names of the
-              converted Core ML model.
+            - TorchScript Models:
+                - If specified, the length of the list must match the number of
+                outputs returned by the PyTorch model.
+                - If ``name`` is specified, it is applied to the output names of the
+                converted Core ML model.
+                - For ``minimum_deployment_target >= ct.target.macOS13``, and with ``compute_precision`` in float 16 precision.
+                If ``dtype`` not specified, the outputs inferred of type float 32
+                defaults to float 16.
+
+            - Torch Exported Models:
+                - The ``outputs`` parameter is not supported. ``outputs`` parameter is inferred from Torch ExportedProgram.
+
 
     classifier_config : ClassifierConfig class (optional)
         The configuration if the MLModel is intended to be a classifier.
@@ -218,7 +252,7 @@ def convert(
         A member of the ``coremltools.target`` enum.
         The value of this parameter determines the type of the model
         representation produced by the converter. To learn about the differences
-        between neural networks and ML programs, see
+        between ML programs and neural networks, see
         `ML Programs <https://coremltools.readme.io/docs/ml-programs>`_.
 
         - The converter produces a neural network (``neuralnetwork``) if:
@@ -236,33 +270,34 @@ def convert(
                                            coremltools.target.tvOS15:
 
         - If neither the ``minimum_deployment_target`` nor the ``convert_to``
-          parameter is specified, the converter produces the neural network
+          parameter is specified, the converter produces an ML program
           model type with as minimum of a deployment target as possible.
         - If this parameter is specified and ``convert_to`` is also specified,
           they must be compatible. The following are examples of invalid values:
           ::
-            # Invalid:
-            convert_to="neuralnetwork", minimum_deployment_target=coremltools.target.iOS15
             # Invalid:
             convert_to="mlprogram", minimum_deployment_target=coremltools.target.iOS14
 
+            # Invalid:
+            convert_to="neuralnetwork", minimum_deployment_target=coremltools.target.iOS15
+
     convert_to : str (optional)
-        Must be one of [``'neuralnetwork'``, ``'mlprogram'``, ``'milinternal'``].
+        Must be one of [``'mlprogram'``, ``'neuralnetwork'``, ``'milinternal'``].
         The value of this parameter determines the type of the model
         representation produced by the converter. To learn about the
-        differences between neural networks and ML programs, see
+        differences between ML programs and neural networks, see
         `ML Programs <https://coremltools.readme.io/docs/ml-programs>`_.
 
+        - ``'mlprogram'`` : Returns an MLModel (``coremltools.models.MLModel``)
+          containing a MILSpec.Program proto, which is the Core ML program format.
+          The model saved from this returned object is executable on iOS15,
+          macOS12, watchOS8, and tvOS15.
         - ``'neuralnetwork'``: Returns an MLModel (``coremltools.models.MLModel``)
           containing a NeuralNetwork proto, which is the original Core ML format.
           The model saved from this returned object is executable either on
           iOS13/macOS10.15/watchOS6/tvOS13 and newer, or on
           iOS14/macOS11/watchOS7/tvOS14 and newer, depending on the layers used
           in the model.
-        - ``'mlprogram'`` : Returns an MLModel (``coremltools.models.MLModel``)
-          containing a MILSpec.Program proto, which is the Core ML program format.
-          The model saved from this returned object is executable on iOS15,
-          macOS12, watchOS8, and tvOS15.
         - ``'milinternal'``: Returns an MIL program object
           (``coremltools.converters.mil.Program``). An MIL program is primarily
           used for debugging and inspection. It can be converted to an MLModel for
@@ -272,7 +307,7 @@ def convert(
              ct.convert(mil_program, convert_to="mlprogram")
 
         - If neither the ``minimum_deployment_target`` nor the ``convert_to``
-          parameter is specified, the converter produces the neural network
+          parameter is specified, the converter produces the ML programs
           model type with as minimum of a deployment target as possible.
 
     compute_precision : coremltools.precision enumeration or ct.transform.FP16ComputePrecision() (optional)
@@ -291,7 +326,7 @@ def convert(
           The above transform iterates through all the ops, looking at each op's
           inputs and outputs. If they are of type float 32, ``cast``
           ops are injected to convert those tensors (also known as `vars`) to
-          type float 16.
+          type float 16. Similarly, int32 vars will also be cast to int16.
 
         - ``coremltools.precision.FLOAT32`` enum: No transform is applied.
 
@@ -354,9 +389,11 @@ def convert(
         Defaults to ``False``.
 
     compute_units: coremltools.ComputeUnit
+        The set of processing units the model can use to make predictions. After
+        conversion, the model is loaded with the provided set of compute units and
+        returned.
 
         An enum with the following possible values.
-
             - ``coremltools.ComputeUnit.ALL``: Use all compute units available, including the
               neural engine.
             - ``coremltools.ComputeUnit.CPU_ONLY``: Limit the model to only use the CPU.
@@ -395,7 +432,7 @@ def convert(
 
             pipeline = ct.PassPipeline()
             pipeline.remove_passes({"common::fuse_conv_batchnorm"})
-            ct.convert(model, pass_pipeline=pipeline)
+            mlmodel = ct.convert(model, pass_pipeline=pipeline)
 
         * To avoid folding too-large ``const`` ops that lead to a large model, set pass option
           as shown in the following example:
@@ -404,7 +441,34 @@ def convert(
 
             pipeline = ct.PassPipeline()
             pipeline.set_options("common::const_elimination", {"skip_const_by_size": "1e6"})
-            ct.convert(model, pass_pipeline=pipeline)
+            mlmodel = ct.convert(model, pass_pipeline=pipeline)
+
+        We also provide a set of predefined pass pipelines that you can directly call.
+
+        * To avoid running all graph pass, you can use:
+
+          .. sourcecode:: python
+
+             mlmodel = ct.convert(model, pass_pipeline=ct.PassPipeline.EMPTY)
+
+        * To only run the cleanup graph passes, like constant_elimination, dead_code_elimination, etc.
+          You can use:
+
+          .. sourcecode:: python
+
+             mlmodel = ct.convert(model, pass_pipeline=ct.PassPipeline.CLEANUP)
+
+        * To convert a source model with sparse weights to a sparse format Core ML model, you can use:
+
+          .. sourcecode:: python
+
+             mlmodel = ct.convert(model, pass_pipeline=ct.PassPipeline.DEFAULT_PRUNING)
+
+        * To convert a source model with palettized weights to a compressed format Core ML model, you can use:
+
+          .. sourcecode:: python
+
+             mlmodel = ct.convert(model, pass_pipeline=ct.PassPipeline.DEFAULT_PALETTIZATION)
 
     Returns
     -------
@@ -443,15 +507,17 @@ def convert(
 
     PyTorch:
 
-        >>> model = torchvision.models.mobilenet_v2()
-        >>> model.eval()
-        >>> example_input = torch.rand(1, 3, 256, 256)
-        >>> traced_model = torch.jit.trace(model, example_input)
+        TorchScript Models:
 
-        >>> input = ct.TensorType(name='input_name', shape=(1, 3, 256, 256))
-        >>> mlmodel = ct.convert(traced_model, inputs=[input])
-        >>> results = mlmodel.predict({"input": example_input.numpy()})
-        >>> print(results['1651']) # 1651 is the node name given by PyTorch's JIT
+            >>> model = torchvision.models.mobilenet_v2()
+            >>> model.eval()
+            >>> example_input = torch.rand(1, 3, 256, 256)
+            >>> traced_model = torch.jit.trace(model, example_input)
+
+            >>> input = ct.TensorType(name='input_name', shape=(1, 3, 256, 256))
+            >>> mlmodel = ct.convert(traced_model, inputs=[input])
+            >>> results = mlmodel.predict({"input": example_input.numpy()})
+            >>> print(results['1651']) # 1651 is the node name given by PyTorch's JIT
 
     See `Conversion Options <https://coremltools.readme.io/docs/neural-network-conversion>`_ for
     more advanced options.
@@ -462,15 +528,25 @@ def convert(
                                      outputs_as_strings,
                                      outputs_as_tensor_or_image_types,
                                      outputs)
+    source_dialect = _determine_source_dialect(model, exact_source)
     exact_target = _determine_target(convert_to, minimum_deployment_target)
-    _validate_conversion_arguments(model, exact_source, inputs, outputs_as_tensor_or_image_types,
-                                   classifier_config, compute_precision,
-                                   exact_target, minimum_deployment_target)
+    _validate_conversion_arguments(
+        model,
+        exact_source,
+        exact_target,
+        inputs,
+        outputs_as_tensor_or_image_types,
+        classifier_config,
+        compute_precision,
+        exact_target,
+        minimum_deployment_target,
+    )
+    need_fp16_cast_pass = _need_fp16_cast_pass(compute_precision, exact_target)
 
     if pass_pipeline is None:
         pass_pipeline = PassPipeline()
-    if not _need_fp16_cast_pass(compute_precision, exact_target):
-        pass_pipeline.remove_passes({"common::add_fp16_cast"})
+    if not need_fp16_cast_pass:
+        pass_pipeline.remove_passes({"common::add_fp16_cast", "common::add_int16_cast"})
     if isinstance(compute_precision, FP16ComputePrecision):
         # For backward compatibility with the `op_selector` param in FP16ComputePrecision.
         pass_pipeline._pass_options["common::add_fp16_cast"] = [
@@ -489,6 +565,12 @@ def convert(
     if specification_version is None:
         specification_version = _set_default_specification_version(exact_target)
 
+    use_default_fp16_io = (
+        specification_version is not None
+        and specification_version >= AvailableTarget.iOS16
+        and need_fp16_cast_pass
+    )
+
     mlmodel = mil_convert(
         model,
         convert_from=exact_source,
@@ -502,8 +584,15 @@ def convert(
         debug=debug,
         specification_version=specification_version,
         main_pipeline=pass_pipeline,
+        use_default_fp16_io=use_default_fp16_io,
     )
 
+    if exact_target == "mlprogram" and mlmodel._input_has_infinite_upper_bound():
+        raise ValueError(
+            "For mlprogram, inputs with infinite upper_bound is not allowed. Please set upper_bound"
+            ' to a positive value in "RangeDim()" for the "inputs" param in ct.convert().'
+        )
+
     if exact_target == 'milinternal':
         return mlmodel  # Returns the MIL program
 
@@ -516,7 +605,7 @@ def convert(
 
     gc.collect()
 
-    mlmodel = _record_build_metadata(mlmodel, exact_source)
+    mlmodel = _record_build_metadata(mlmodel, exact_source, source_dialect=source_dialect)
 
     return mlmodel
 
@@ -539,7 +628,7 @@ def _need_fp16_cast_pass(
         raise ValueError(f"Invalid value of the argument 'compute_precision': {compute_precision}")
 
 
-def _set_default_specification_version(target):
+def _set_default_specification_version(target) -> Optional[AvailableTarget]:
     if target == "neuralnetwork":
         return _LOWEST_ALLOWED_SPECIFICATION_VERSION_FOR_NEURALNETWORK
     elif target == "mlprogram":
@@ -625,18 +714,20 @@ def _validate_outputs_argument(outputs):
                 return output_names, outputs
 
 
-def _validate_conversion_arguments(model,
-                                   exact_source,
-                                   inputs,
-                                   outputs,
-                                   classifier_config,
-                                   compute_precision,
-                                   convert_to,
-                                   minimum_deployment_target,
-                                   ):
+def _validate_conversion_arguments(
+    model,
+    exact_source,
+    exact_target,
+    inputs,
+    outputs,
+    classifier_config,
+    compute_precision,
+    convert_to,
+    minimum_deployment_target,
+):
     """
     Validate and process model, inputs, classifier_config based on
-    `exact_source` (which cannot be `auto`)
+    `exact_source` (which cannot be `auto`) and `exact_target`.
     """
 
     def raise_if_duplicated(input_list):
@@ -672,10 +763,10 @@ def _validate_conversion_arguments(model,
 
         # get flattened inputs
         flat_inputs = _flatten_list(inputs)
-        for t in flat_inputs:
-            if not isinstance(t, InputType):
+        for flat_input in flat_inputs:
+            if not isinstance(flat_input, InputType):
                 raise ValueError("inputs must be a list of type ct.TensorType or ct.ImageType")
-            if t.dtype == types.fp16:
+            if flat_input.dtype == types.fp16:
                 if not (
                     minimum_deployment_target is not None
                     and minimum_deployment_target >= AvailableTarget.iOS16
@@ -685,6 +776,24 @@ def _validate_conversion_arguments(model,
                         "target >= iOS16/macOS13/watchOS9/tvOS16"
                     )
 
+    if exact_target == "mlprogram":
+        err_msg_infinite_bound = (
+            "For mlprogram, inputs with infinite upper_bound is not allowed. Please set upper_bound"
+            ' to a positive value in "RangeDim()" for the "inputs" param in ct.convert().'
+        )
+        if inputs is not None:
+            for flat_input in _flatten_list(inputs):
+                tensor_shapes: List[Optional[Shape]] = (
+                    flat_input.shape.shapes
+                    if isinstance(flat_input.shape, EnumeratedShapes)
+                    else [flat_input.shape]
+                )
+                for tensor_shape in tensor_shapes:
+                    if tensor_shape is not None:
+                        for shape in tensor_shape.shape:
+                            if isinstance(shape, RangeDim) and shape.upper_bound < 0:
+                                raise ValueError(err_msg_infinite_bound)
+
     if outputs is not None:
         for t in outputs:
             if t.dtype == types.fp16:
@@ -731,16 +840,45 @@ def _validate_conversion_arguments(model,
             raise ValueError("Input should be a list of TensorType or ImageType")
 
     elif exact_source == "pytorch":
-        if inputs is None:
-            raise ValueError('Expected argument for pytorch "inputs" not provided')
+        if _HAS_TORCH_EXPORT_API and isinstance(model, ExportedProgram):
+            if model.dialect != "EDGE":
+                raise NotImplementedError(
+                    f"Conversion for models with only EDGE dialect is supported/tested. Provided Dialect: {model.dialect}"
+                )
 
-        raise_if_duplicated(flat_inputs)
-        if inputs is not None and not all(
-            [isinstance(_input, InputType) for _input in flat_inputs]
-        ):
-            raise ValueError(
-                "Input should be a list/tuple (or nested lists/tuples) of TensorType or ImageType"
-            )
+            # TODO: rdar://115845792 ([Executorch] Handle user provided inputs/outputs in the convert API)
+            if inputs is not None:
+                raise AssertionError("'inputs' argument should be None for ExportedProgram")
+
+            if outputs is not None:
+                raise AssertionError("'outputs' argument should be None for ExportedProgram")
+
+        else:
+            is_torch_load_successful = False
+            try:
+                try_load_torchscript(model)
+                is_torch_load_successful = True
+            except:
+                pass
+            if is_torch_load_successful:
+                if inputs is None:
+                    raise ValueError(
+                        'Expected argument "inputs" for TorchScript models not provided'
+                    )
+
+                raise_if_duplicated(flat_inputs)
+                if inputs is not None and not all(
+                    [isinstance(_input, InputType) for _input in flat_inputs]
+                ):
+                    raise ValueError(
+                        "Input should be a list/tuple (or nested lists/tuples) of TensorType or ImageType"
+                    )
+            else:
+                raise TypeError(
+                    "@model must either be a TorchScript object (or .pt or .pth file) or an ExportedProgram object (if using torch.export based API), received: {}".format(
+                        type(model)
+                    )
+                )
 
     elif exact_source == "milinternal":
         if not isinstance(model, Program):
@@ -749,6 +887,19 @@ def _validate_conversion_arguments(model,
             )
 
 
+def _determine_source_dialect(model, exact_source):
+
+    source_dialect = None
+    if exact_source == "pytorch":
+
+        if _HAS_TORCH_EXPORT_API and isinstance(model, ExportedProgram):
+            return f"TorchExport::{model.dialect}"
+        else:
+            return "TorchScript"
+
+    return source_dialect
+
+
 def _determine_source(model, source,
                       output_names,
                       outputs_as_tensor_or_image_types,
@@ -787,9 +938,13 @@ def _determine_source(model, source,
             pass
 
     if source == "auto" and _HAS_TORCH:
+
+        if _HAS_TORCH_EXPORT_API and isinstance(model, ExportedProgram):
+            return "pytorch"
+
         is_torch_load_successful = False
         try:
-            pytorch_load(model)
+            try_load_torchscript(model)
             is_torch_load_successful = True
         except:
             pass
@@ -826,6 +981,15 @@ def _determine_target(convert_to, minimum_deployment_target):
     """
     Infer the precise backend target, which could be one of ``milinternal``, ``neuralnetwork`` or ``mlprogram``
     """
+    if minimum_deployment_target is None and convert_to is None:
+        logger.warning(
+            "When both 'convert_to' and 'minimum_deployment_target' not specified, "
+            "'convert_to' is set to \"mlprogram\" and 'minimum_deployment_targer' is set to "
+            "ct.target.iOS15 (which is same as ct.target.macOS12). "
+            "Note: the model will not run on systems older than iOS15/macOS12/watchOS8/tvOS15. "
+            "In order to make your model run on older system, please set the 'minimum_deployment_target' to iOS14/iOS13. "
+            "Details please see the link: https://coremltools.readme.io/docs/unified-conversion-api#target-conversion-formats"
+        )
     if minimum_deployment_target is not None:
         if convert_to == "mlprogram" and minimum_deployment_target < AvailableTarget.iOS15:
             raise ValueError(
@@ -844,7 +1008,7 @@ def _determine_target(convert_to, minimum_deployment_target):
         return convert_to
     else:
         if minimum_deployment_target is None:
-            return "neuralnetwork"
+            return "mlprogram"
         elif minimum_deployment_target <= AvailableTarget.iOS14:
             return "neuralnetwork"
         else:
@@ -856,6 +1020,12 @@ def _get_metadata_from_mlmodel(mlmodel):
     src_pkg_version = mlmodel.user_defined_metadata[_METADATA_SOURCE]
     coremltools_version = mlmodel.user_defined_metadata[_METADATA_VERSION]
 
+    src_dialect = (
+        None
+        if _METADATA_SOURCE_DIALECT not in mlmodel.user_defined_metadata
+        else mlmodel.user_defined_metadata[_METADATA_SOURCE_DIALECT]
+    )
+
     src_pkg_version_list = src_pkg_version.split("==")
     if len(src_pkg_version_list) == 0:
         src_pkg, pkg_ver = None, None
@@ -872,10 +1042,13 @@ def _get_metadata_from_mlmodel(mlmodel):
     if src_pkg is not None and pkg_ver is not None:
         build_info['coremltools-component-' + src_pkg] = str(pkg_ver)
 
+    if src_dialect is not None:
+        build_info["coremltools-source-dialect"] = src_dialect
+
     return build_info
 
 
-def _record_build_metadata(mlmodel, exact_source):
+def _record_build_metadata(mlmodel, exact_source, source_dialect=None):
     # recording metadata: coremltools version, source framework and version
     if exact_source in {"tensorflow", "tensorflow2"} and (_HAS_TF_1 or _HAS_TF_2):
         src_pkg_version = "tensorflow=={0}".format(tf.__version__)
@@ -889,6 +1062,9 @@ def _record_build_metadata(mlmodel, exact_source):
     mlmodel.user_defined_metadata[_METADATA_SOURCE] = src_pkg_version
     mlmodel.user_defined_metadata[_METADATA_VERSION] = _ct_version
 
+    if source_dialect is not None:
+        mlmodel.user_defined_metadata[_METADATA_SOURCE_DIALECT] = source_dialect
+
     build_info = _get_metadata_from_mlmodel(mlmodel)
 
     mlmodel._set_build_info_mil_attributes(build_info)
