@@ -4,6 +4,8 @@
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 import os
+import warnings
+from typing import Optional
 
 import numpy as np
 
@@ -21,7 +23,7 @@ from coremltools.converters.mil.backend.mil.helper import (
 from coremltools.converters.mil.backend.nn.load import _set_optional_inputs
 from coremltools.converters.mil.input_types import EnumeratedShapes, ImageType, RangeDim, TensorType
 from coremltools.converters.mil.mil import Builder as mb
-from coremltools.converters.mil.mil import Function, mil_list, types
+from coremltools.converters.mil.mil import Function, Program, mil_list, types
 from coremltools.converters.mil.mil.ops.registry import SSAOpRegistry
 from coremltools.converters.mil.mil.types.symbolic import any_symbolic, any_variadic, is_symbolic
 from coremltools.models.neural_network.flexible_shape_utils import (
@@ -41,7 +43,8 @@ from ..backend_helper import _get_colorspace_enum, _validate_image_input_output_
 
 try:
     from coremltools.libmilstoragepython import _BlobStorageWriter as BlobWriter
-except:
+except Exception as e:
+    logger.warning(f"Fail to import BlobWriter from libmilstoragepython. {e}")
     BlobWriter = None
 
 
@@ -146,6 +149,8 @@ def translate_generic_op(op, parameters, blob_writer, literal_params=[]):
         attr_dict["weights"] = create_list_scalarvalue(weights, str)
         attr_dict["description"] = create_scalar_value(description)
 
+    attr_dict["name"] = create_scalar_value(op.name)
+
     return pm.Operation(
         type=op_type,
         blocks=blocks,
@@ -278,21 +283,17 @@ def _add_classify_op(prog, classifier_config):
         return out[0].name, out[1].name
 
 
-def load(prog, weights_dir, resume_on_errors=False, specification_version=_SPECIFICATION_VERSION_IOS_15, **kwargs):
+def _pymil_to_milproto(
+    prog: Program,
+    weights_dir: str,
+    specification_version: Optional[int] = _SPECIFICATION_VERSION_IOS_15,
+) -> pm.Program:
+    """
+    Convert a pymil program into mil proto.
+    """
     if BlobWriter is None:
         raise RuntimeError("BlobWriter not loaded")
-    if "main" not in prog.functions:
-        raise ValueError("main function not found in program")
 
-    # if user has specified "ClassifierConfig", then add the "classify" op to the prog
-    classifier_config = kwargs.get("classifier_config", None)
-    predicted_feature_name = None
-    predicted_probabilities_name = None
-    if classifier_config is not None:
-        predicted_feature_name, predicted_probabilities_name = _add_classify_op(prog, classifier_config)
-
-    input_types = prog.main_input_types
-    output_types = prog.main_output_types
     weight_path = os.path.join(weights_dir, _WEIGHTS_FILE_NAME)
     blob_writer = BlobWriter(weight_path)
 
@@ -306,6 +307,33 @@ def load(prog, weights_dir, resume_on_errors=False, specification_version=_SPECI
         version=1,
         functions=function_protos,
     )
+    return proto
+
+
+def load(
+    prog: Program,
+    weights_dir: str,
+    resume_on_errors: Optional[bool] = False,
+    specification_version: Optional[int] = _SPECIFICATION_VERSION_IOS_15,
+    **kwargs,
+):
+    if "main" not in prog.functions:
+        raise ValueError("main function not found in program")
+
+    # if user has specified "ClassifierConfig", then add the "classify" op to the prog
+    classifier_config = kwargs.get("classifier_config", None)
+    predicted_feature_name = None
+    predicted_probabilities_name = None
+    if classifier_config is not None:
+        predicted_feature_name, predicted_probabilities_name = _add_classify_op(
+            prog, classifier_config
+        )
+
+    # convert pymil program into mil proto
+    proto = _pymil_to_milproto(prog, weights_dir, specification_version)
+
+    input_types = prog.main_input_types
+    output_types = prog.main_output_types
 
     desc = kwargs.get("model_description", None)
     if desc and not isinstance(desc, ml.ModelDescription):
@@ -329,8 +357,14 @@ def load(prog, weights_dir, resume_on_errors=False, specification_version=_SPECI
             image_input_names[input_type.name] = input_type
             # error checking for input(s) marked as images
             if input_type.name not in list(prog.functions["main"].inputs.keys()):
-                msg = "Provided image input '{}' is not one of the inputs of the MIL program"
-                raise ValueError(msg.format(input_type.name))
+                raise ValueError(
+                    f"Provided image input '{input_type.name}' is not one of the inputs of the MIL program"
+                )
+        if input_type.name is None:
+            raise ValueError(
+                'Fail to auto-determine the input name. Please specify the "name" '
+                'parameter when use "inputs" in ct.convert().'
+            )
         input_shape_map[input_type.name] = input_type
 
     for name, var in prog.functions["main"].inputs.items():
@@ -420,7 +454,12 @@ def load(prog, weights_dir, resume_on_errors=False, specification_version=_SPECI
                     # Classifier outputs are set up separately, so default to fp32 for now.
                     dataType = ft.ArrayFeatureType.ArrayDataType.FLOAT32
 
-                array_type = ft.ArrayFeatureType(shape=None, dataType=dataType)
+                output_shape = (
+                    None
+                    if any_symbolic(var.shape) or types.is_primitive(var.sym_type)
+                    else var.shape
+                )
+                array_type = ft.ArrayFeatureType(shape=output_shape, dataType=dataType)
                 output_feature_type.multiArrayType.CopyFrom(array_type)
                 output_features.append(ml.FeatureDescription(name=var.name, type=output_feature_type))
         elif (types.is_dict(var.sym_type)):
@@ -457,6 +496,11 @@ def load(prog, weights_dir, resume_on_errors=False, specification_version=_SPECI
     model.mlProgram.CopyFrom(proto)
 
     # Set symbolic shapes
+    default_lower_bound = 1
+    default_upper_bound = (
+        default_lower_bound + 1 if kwargs.get("convert_to", None) == "mlprogram" else -1
+    )
+    default_bound_used = False
     for input_name in symbolic_inputs:
         input_type = input_shape_map.get(input_name, None)
 
@@ -480,13 +524,15 @@ def load(prog, weights_dir, resume_on_errors=False, specification_version=_SPECI
                 if isinstance(H, RangeDim):
                     img_range.add_height_range((H.lower_bound, H.upper_bound))
                 elif is_symbolic(H):
-                    img_range.add_height_range((1, -1))
+                    img_range.add_height_range((default_lower_bound, default_upper_bound))
+                    default_bound_used = True
                 else:
                     img_range.add_height_range((H, H))
                 if isinstance(W, RangeDim):
                     img_range.add_width_range((W.lower_bound, W.upper_bound))
                 elif is_symbolic(W):
-                    img_range.add_width_range((1, -1))
+                    img_range.add_width_range((default_lower_bound, default_upper_bound))
+                    default_bound_used = True
                 else:
                     img_range.add_width_range((W, W))
 
@@ -506,8 +552,9 @@ def load(prog, weights_dir, resume_on_errors=False, specification_version=_SPECI
                         lb.append(s.lower_bound)
                         ub.append(s.upper_bound)
                     elif is_symbolic(s):
-                        lb.append(1)
-                        ub.append(-1)
+                        lb.append(default_lower_bound)
+                        ub.append(default_upper_bound)
+                        default_bound_used = True
                     else:
                         lb.append(s)
                         ub.append(s)
@@ -520,8 +567,9 @@ def load(prog, weights_dir, resume_on_errors=False, specification_version=_SPECI
             ub = []
             for s in sym_type.get_shape():
                 if is_symbolic(s):
-                    lb.append(1)
-                    ub.append(-1)
+                    lb.append(default_lower_bound)
+                    ub.append(default_upper_bound)
+                    default_bound_used = True
                 else:
                     lb.append(s)
                     ub.append(s)
@@ -529,6 +577,22 @@ def load(prog, weights_dir, resume_on_errors=False, specification_version=_SPECI
                 model, input_name, lower_bounds=lb, upper_bounds=ub
             )
 
+    if default_bound_used and kwargs.get("convert_to", None) == "mlprogram":
+        warnings.warn(
+            "Some dimensions in the input shape are unknown, hence they are set to flexible ranges "
+            f"with lower bound and default value = {default_lower_bound}, and upper bound = "
+            f"{default_upper_bound}. To set different values for the default shape and upper bound, "
+            "please use the ct.RangeDim() method as described here: "
+            "https://coremltools.readme.io/docs/flexible-inputs#set-the-range-for-each-dimension.",
+            UserWarning,
+        )
+        convert_from = kwargs.get("convert_from", None)
+        if convert_from is not None and convert_from.startswith("tensorflow"):
+            warnings.warn(
+                'There is "None" dim in TF input placeholder. Please consider specifying '
+                'input shapes by using the "inputs" param in ct.convert().'
+            )
+
     # Set optional inputs
     _set_optional_inputs(model, input_types)
 
