@@ -8,16 +8,22 @@ import os
 import numpy as np
 
 from coremltools import _logger as logger
-from coremltools.converters.mil._deployment_compatibility import \
-    AvailableTarget as _target
+from coremltools.converters.mil._deployment_compatibility import AvailableTarget as _target
+from coremltools.converters.mil.backend.mil import helper
 from coremltools.converters.mil.mil import Block
 from coremltools.converters.mil.mil import Builder as mb
-from coremltools.converters.mil.mil import (Function, ListVar, Placeholder,
-                                            Program, TupleInputType, Var,
-                                            mil_list, types)
+from coremltools.converters.mil.mil import (
+    Function,
+    ListVar,
+    Placeholder,
+    Program,
+    TupleInputType,
+    Var,
+    mil_list,
+    types,
+)
 from coremltools.converters.mil.mil.block import curr_block
-from coremltools.converters.mil.mil.ops.registry import \
-    SSAOpRegistry as _SSAOpRegistry
+from coremltools.converters.mil.mil.ops.registry import SSAOpRegistry as _SSAOpRegistry
 from coremltools.proto import MIL_pb2 as pm
 from coremltools.proto import Model_pb2 as ml
 
@@ -25,7 +31,8 @@ from .helper import proto_to_types
 
 try:
     from coremltools.libmilstoragepython import _BlobStorageReader as BlobReader
-except:
+except Exception as e:
+    logger.warning(f"Fail to import BlobReader from libmilstoragepython. {e}")
     BlobReader = None
 
 
@@ -107,14 +114,18 @@ def _load_file_value(context, filevalue_spec, dtype):
         context.blob_reader_from_filename[filename] = blob_reader
 
     if dtype == types.uint8:
-        np_value = np.array(blob_reader.read_uint8_data(offset), np.uint8)
+        np_value = blob_reader.read_uint8_data(offset)
     elif dtype == types.int8:
-        np_value = np.array(blob_reader.read_int8_data(offset), np.int8)
+        np_value = blob_reader.read_int8_data(offset)
+    elif dtype == types.uint16:
+        np_value = blob_reader.read_uint16_data(offset)
+    elif dtype == types.int16:
+        np_value = blob_reader.read_int16_data(offset)
     elif dtype == types.fp16:
-        np_value_uint16 = np.array(blob_reader.read_fp16_data(offset), np.uint16)
+        np_value_uint16 = blob_reader.read_fp16_data(offset)
         np_value = np.frombuffer(np_value_uint16.tobytes(), np.float16)
     elif dtype == types.fp32:
-        np_value = np.array(blob_reader.read_float_data(offset), np.float32)
+        np_value = blob_reader.read_float_data(offset)
     else:
         raise ValueError("Invalid dtype for blob file value type")
 
@@ -141,7 +152,7 @@ def _load_value(context, value_spec):
         else:
             value = _load_file_value(context, value_spec.blobFileValue, dtype)
 
-        if dtype in (types.fp16, types.int8, types.uint8, types.uint32):
+        if dtype in helper.IMMEDIATE_VALUE_TYPES_IN_BYTES:
             value = np.frombuffer(value, types.nptype_from_builtin(dtype)).reshape(
                 shape
             )
@@ -242,20 +253,23 @@ def _set_inputs_for_control_flow_op(inputs, blocks, op_type):
         inputs["_false_fn"] = _dummy_false_fn
 
 
+def _load_const_op(context, op_spec):
+    inputs = {k: _load_value(context, v) for k, v in op_spec.attributes.items()}
+    pymil_var = getattr(mb, op_spec.type)(**inputs)
+    context.register_var_with_name(op_spec.outputs[0].name, pymil_var)
+
+
 def _load_operation(context, op_spec):
     if not isinstance(op_spec, pm.Operation):
         raise TypeError("Invalid Operation spec object")
 
     op_type = op_spec.type
-    if op_type == "const" or op_type.startswith("constexpr_"):
+    if op_type == "const" or "constexpr_" in op_type:
         if op_spec.blocks:
             raise ValueError("const / constexpr operation can't have any block")
         if op_spec.inputs:
             raise ValueError("const / constexpr operation can't have any input")
-
-        inputs = {k: _load_value(context, v) for k, v in op_spec.attributes.items()}
-        pymil_var = getattr(mb, op_type)(**inputs)
-        context.register_var_with_name(op_spec.outputs[0].name, pymil_var)
+        _load_const_op(context, op_spec)
 
     else:
         if op_type == "custom_layer":
@@ -263,9 +277,6 @@ def _load_operation(context, op_spec):
                 "Loading Custom Layer operation not yet implemented"
             )
 
-        if op_spec.attributes:
-            raise ValueError("Attributes on operation not supported")
-
         # The conversion steps of an operation proto -> PyMIL operation are as following:
 
         # (i)   Convert the input arguments:
@@ -289,7 +300,12 @@ def _load_operation(context, op_spec):
         # (iv)  Set the outer_op for control flow
         #       Once the operation is created, we replace the dummy outer_op with the legit one, to make it a valid PyMIL program
 
-        inputs = {}
+        attrs = list(op_spec.attributes.items())
+        if len(attrs) > 0:
+            if len(attrs) != 1 or attrs[0][0] != "name":
+                raise ValueError("\"name\" is the only supported attribute for operation")
+        inputs = {k: _load_value(context, v) for k, v in op_spec.attributes.items()}
+
         for param_name, argument in op_spec.inputs.items():
             vars = []
             for binding in argument.arguments:
@@ -395,17 +411,10 @@ def _load_function(context, func_spec, spec_version):
     return pymil_func
 
 
-def load(model_spec, specification_version, file_weights_dir="", **kwargs):
-    if not isinstance(model_spec, ml.Model):
-        raise TypeError("Invalid Model sepc object")
-
-    if specification_version < model_spec.specificationVersion:
-        raise ValueError("specification_version must be greater or equal to the input model spec version")
-
-    if model_spec.WhichOneof("Type") != "mlProgram":
-        raise ValueError("Only MIL proto based mlmodels can be loaded")
-
-    program_spec = model_spec.mlProgram
+def load_mil_proto(program_spec, specification_version, file_weights_dir=""):
+    """
+    Load in-memory Proto specification of MILSpec.Program(.Proto) object to PyMIL
+    """
     if not isinstance(program_spec, pm.Program):
         raise TypeError("Invalid Program spec object")
 
@@ -427,3 +436,24 @@ def load(model_spec, specification_version, file_weights_dir="", **kwargs):
             raise ValueError("Invalid attribute for program")
 
     return pymil_program
+
+
+def load(model_spec, specification_version, file_weights_dir="", **kwargs):
+    """
+    Load in-memory Proto specification of Model(.Proto) object to PyMIL
+
+    Set force_spec_version to force override the spec version.
+    """
+    if not isinstance(model_spec, ml.Model):
+        raise TypeError("Invalid Model sepc object")
+
+    if specification_version < model_spec.specificationVersion:
+        if not kwargs.get("force_spec_version", False):
+            raise ValueError(
+                "specification_version must be greater or equal to the input model spec version"
+            )
+
+    if model_spec.WhichOneof("Type") != "mlProgram":
+        raise ValueError("Only MIL proto based mlmodels can be loaded")
+
+    return load_mil_proto(model_spec.mlProgram, specification_version, file_weights_dir)
