@@ -527,6 +527,11 @@ class TestConvolution(TensorFlowBaseTest):
         TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen((1, 80, 40, 1), rand_min=-10, rand_max=10)],
+            inputs_for_conversion=[
+                ct.TensorType(
+                    shape=(1, ct.RangeDim(upper_bound=80), ct.RangeDim(upper_bound=80), 1)
+                )
+            ],
             compute_unit=compute_unit,
             backend=backend,
         )
@@ -862,6 +867,36 @@ class TestFlatten(TensorFlowBaseTest):
             backend=backend,
         )
 
+class TestMasking(TensorFlowBaseTest):
+    @pytest.mark.parametrize(
+        "compute_unit, backend, rank, mask_value, is_masked",
+        itertools.product(
+            compute_units,
+            backends,
+            [2, 5],
+            [0, 0.4],
+            [False, True],
+        ),
+    )
+    def test(self, compute_unit, backend, rank, mask_value, is_masked):
+        shape = np.random.randint(low=2, high=4, size=rank)
+        model = tf.keras.Sequential(
+            [
+                tf.keras.layers.Masking(
+                    batch_input_shape=shape,
+                    mask_value=mask_value,
+                )
+            ]
+        )
+        input_value = random_gen(shape, -10, 10)
+        if is_masked:
+            input_value[:, 1] = mask_value
+        TensorFlowBaseTest.run_compare_tf_keras(
+            model,
+            [input_value],
+            compute_unit=compute_unit,
+            backend=backend,
+        )
 
 class TestLambda(TensorFlowBaseTest):
     @pytest.mark.parametrize(
@@ -1038,9 +1073,14 @@ class TestNormalization(TensorFlowBaseTest):
     def test_layer_normalization(self, compute_unit, backend, rank, axis, epsilon, dynamic):
         shape = np.random.randint(low=2, high=4, size=rank)
         keras_shape = shape.tolist()
+        inputs_for_conversion = None
 
         if dynamic:
             keras_shape[0] = None
+            if backend[0] == "mlprogram":
+                inputs_for_conversion = [
+                    ct.TensorType(shape=[ct.RangeDim(upper_bound=4)] + keras_shape[1:])
+                ]
 
         model = tf.keras.Sequential(
             [
@@ -1052,6 +1092,7 @@ class TestNormalization(TensorFlowBaseTest):
         TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-100, rand_max=100)],
+            inputs_for_conversion=inputs_for_conversion,
             compute_unit=compute_unit,
             backend=backend,
         )
@@ -1348,6 +1389,11 @@ class TestRecurrent(TensorFlowBaseTest):
         "compute_unit, backend", itertools.product(compute_units, backends)
     )
     def test_lstm_dynamic_batch(self, compute_unit, backend):
+        if backend == ("mlprogram", "fp16"):
+            pytest.xfail(
+                "rdar://116060011: re-activate coremltools tests blocked by Core ML regressions"
+            )
+
         input_shape = (1, 1280)
         inp = tf.keras.layers.Input(shape=input_shape)
         out, hn, cn = tf.keras.layers.LSTM(512,
@@ -1690,9 +1736,16 @@ class TestUpSampling(TensorFlowBaseTest):
             kwargs = {"data_format": data_format}
             shape = np.random.randint(low=2, high=4, size=5)
             keras_shape = np.copy(shape).tolist()
-            # not support upsampling3D with dynamic input shape, since 6D tensors are produced in that case
             if dynamic:
-                return
+                pytest.skip(
+                    "upsampling3D with dynamic input shape is not supported, since 6D tensors are produced in that case"
+                )
+
+        inputs_for_conversion = None
+        if backend[0] == "mlprogram" and dynamic:
+            inputs_for_conversion = [
+                ct.TensorType(shape=[dim or ct.RangeDim(upper_bound=10) for dim in keras_shape])
+            ]
 
         model = tf.keras.Sequential(
             [op(batch_input_shape=keras_shape, size=upsample_factor, **kwargs)]
@@ -1700,6 +1753,7 @@ class TestUpSampling(TensorFlowBaseTest):
         spec = TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
+            inputs_for_conversion=inputs_for_conversion,
             compute_unit=compute_unit,
             backend=backend,
         )[0]
