@@ -5,12 +5,14 @@
 
 import copy
 from collections import Counter, OrderedDict
+from typing import Tuple
 
-from coremltools import _OPSET, _logger as logger
-from coremltools.converters.mil._deployment_compatibility import \
-    AvailableTarget as _target
+from coremltools import _OPSET
+from coremltools import _logger as logger
+from coremltools.converters.mil._deployment_compatibility import AvailableTarget as _target
 
 from . import SPACES, types
+from .operation import Operation
 from .types.symbolic import is_symbolic, k_used_symbols
 from .var import ComplexVar, InternalVar, Var
 from .visitors.dot_visitor import DotVisitor
@@ -356,13 +358,11 @@ class Block:
             if not isinstance(v, (Var, tuple)):
                 continue
             vs = [v] if isinstance(v, Var) else v
-            for s in vs:
-                if not self.is_var_visible_in_block(s, upto_op_with_id=idx):
+            for v in vs:
+                if not self.is_var_visible_in_block(v, upto_op_with_id=idx):
                     before_op_name = before_op.name if before_op is not None else "None"
                     msg = "Op '{}' input {}={} is not in scope of {} before {}"
-                    raise ValueError(
-                        msg.format(new_op.name, k, s.name, self.name, before_op_name)
-                    )
+                    raise ValueError(msg.format(new_op.name, k, v.name, self.name, before_op_name))
 
         # add new_op
         if before_op is None:
@@ -445,6 +445,8 @@ class Block:
             old_var.consuming_blocks.remove(self)
             # Ensure output name is consistent
             if isinstance(self, Function):
+                if new_var in self.inputs.values() and new_var.name != old_var.name:
+                    raise ValueError("It is not allowed to modify function inputs name.")
                 new_var.name = old_var.name
 
     def try_replace_uses_of_var_after_op(
@@ -452,6 +454,7 @@ class Block:
         anchor_op,
         old_var,
         new_var,
+        end_op=None,
         no_check_var_types=False,
         no_check_var_visibility=False,
     ):
@@ -459,6 +462,7 @@ class Block:
         :param anchor_op: Operation
         :param old_var: Var
         :param new_var: Var
+        :param end_op: Operation
         :param no_check_var_types: bool
         :param no_check_var_visibility: bool
         :return: True if the old_var can be replaced by new_var. False otherwsie.
@@ -474,6 +478,7 @@ class Block:
 
         self.replace_uses_of_var_after_op(
             anchor_op=anchor_op,
+            end_op=end_op,
             old_var=old_var,
             new_var=new_var,
             no_check_var_types=no_check_var_types,
@@ -892,3 +897,26 @@ class Function(Block):
             s += self.indented_str(SPACES)
             s += "}\n"
         return s
+
+    def get_max_opset_version_and_op(self) -> Tuple[_target, Operation]:
+        """
+        Find the max opset version among all operations in the function.
+        Returns the opset version Enum and the corresponding op.
+        """
+        max_opset_version = _target.iOS13
+        op_with_max_opset_version = None
+
+        def update_max_opset_version_block(block):
+            nonlocal max_opset_version
+            nonlocal op_with_max_opset_version
+            for op in list(block.operations):
+                for b in op.blocks:
+                    update_max_opset_version_block(b)
+                if not hasattr(op, "_op_variants") or not isinstance(op._op_variants, dict):
+                    continue
+                if op.opset_version > max_opset_version:
+                    max_opset_version = op.opset_version
+                    op_with_max_opset_version = op
+
+        update_max_opset_version_block(self)
+        return max_opset_version, op_with_max_opset_version
