@@ -148,11 +148,12 @@ class InputSpec:
             input_type = self.input_types[name]
             # Check constness
             # Don't check InternalInputType (so _const_symbolic can work)
-            if input_type.const and \
-                not isinstance(input_type, InternalInputType) \
-                and var.val is None:
-                msg = msg_prefix + \
-                    'Input {} must be const at compile time'
+            if (
+                input_type.const
+                and not isinstance(input_type, InternalInputType)
+                and not var.is_descendant_of_const
+            ):
+                msg = msg_prefix + "Input {} must be const at compile time"
                 raise ValueError(msg.format(name), name, var.name)
 
             if not isinstance(var, InternalVar) and \
@@ -275,7 +276,7 @@ class TensorInputType(_InputType):
         super().__init__(**kwargs)
 
     def _is_compatible(self, v):
-        result = types.is_scalar(v.dtype) or types.is_tensor(v.dtype)
+        result = types.is_scalar(v.sym_type) or types.is_tensor(v.sym_type)
         result = result and (v.dtype in self.type_domain)
         return result
 
@@ -285,7 +286,7 @@ class TensorInputType(_InputType):
 
     @type_domain.setter
     def type_domain(self, val):
-        msg = "type_domain must be a tuple of builtin types"
+        msg = f"type_domain {val} must be a tuple of builtin types"
         if not isinstance(val, tuple) or any(map(lambda t: t not in _SUPPORT_TYPES, val)):
             raise ValueError(msg)
         self._type_domain = val
@@ -308,9 +309,6 @@ class ListInputType(_InputType):
     """
     ListInputType allows inputs of type types.list
     """
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-
     def _is_compatible(self, v):
         return types.is_list(v.sym_type)
 
@@ -325,14 +323,9 @@ class ListOrTensorInputType(_InputType):
     (1) MIL tensor
     (2) python list/tuple of MIL tensors
     """
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-
     def _is_compatible(self, v):
         return (
-            types.is_list(v.sym_type)
-            or types.is_scalar(v.dtype)
-            or types.is_tensor(v.dtype)
+            types.is_list(v.sym_type) or types.is_scalar(v.sym_type) or types.is_tensor(v.sym_type)
         )
 
     @property
@@ -344,9 +337,6 @@ class TupleInputType(_InputType):
     """
     TupleInputType specifies input types of python list/tuple of MIL tensors.
     """
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-
     def _is_compatible(self, v):
         # We don't check the detail types within the tuple.
         return isinstance(v, (tuple, list))
@@ -362,10 +352,6 @@ class InternalInputType(_InputType):
     It allows ops to take, for example, python primitive types, instead of
     only the builtin types.
     """
-
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-
     def _is_compatible(self, v):
         return True  # skip type check by default for InternalInputType.
 
@@ -374,9 +360,5 @@ class PyFunctionInputType(InternalInputType):
     """
     Native python function.
     """
-
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-
     def _is_compatible(self, v):
         return callable(v.val)
