@@ -20,6 +20,37 @@ from coremltools.converters.mil.mil.types.symbolic import is_symbolic
 
 MAX_SIZE_CONSTANT_FOLDING = 1024 * 1024 / 4 # When a fp32 const takes over 1MB, we won't create a const op for that
 
+class ConvPoolingTypeInferenceCache(dict):
+    """
+    An utility class to cache the shape inference of ``conv`` and ``pool`` op.
+    The cache mechanism makes sure ops with the same input shape (symbolic also),
+    and params (``pad, stride, kernel``) would produce the same output shape.
+    """
+    @staticmethod
+    def get_cache_key(
+        input_shape: Tuple[int],
+        pad_type: str,
+        pad: Tuple[int],
+        strides: Tuple[int],
+        kernel: Tuple[int],
+        ceil_mode: bool,
+    ) -> Tuple[Tuple]:
+        return (
+            ("input_shape", input_shape),
+            ("pad_type", pad_type),
+            ("pad", pad),
+            ("strides", strides),
+            ("kernel", kernel),
+            ("ceil_mode", ceil_mode),
+        )
+
+    def __setitem__(self, key, value):
+        if key in self:
+            raise ValueError(f"cache key {key} already exisit.")
+        return dict.__setitem__(self, key, value)
+
+CONV_POOLING_TYPE_INFERENCE_CACHE = ConvPoolingTypeInferenceCache()
+
 def broadcast_shapes(shape_x, shape_y):
     """
     Check and broadcast given input shapes.
@@ -72,6 +103,30 @@ def broadcast_shapes(shape_x, shape_y):
     return tuple(ret_shapes)
 
 
+def infer_type_with_broadcast(typea, typeb, primitive_type):
+    """
+    Given 2 primitive types `typea` and `typeb`, and their promotion `primitive_type`,
+    return the type after broadcast
+    """
+
+    # broadcast
+    if not types.is_tensor(typea) and not types.is_tensor(typeb):
+        # both typea and typeb are not tensors
+        return primitive_type
+    if types.is_tensor(typea) and not types.is_tensor(typeb):
+        # a is tensor, b is not
+        return types.tensor(primitive_type, typea.get_shape())
+    if not types.is_tensor(typea) and types.is_tensor(typeb):
+        # a is not tensor, b is
+        return types.tensor(primitive_type, typeb.get_shape())
+
+    # both a, b are tensors
+    shapea = list(typea.get_shape())
+    shapeb = list(typeb.get_shape())
+    ret_shape = broadcast_shapes(shapea, shapeb)
+    return types.tensor(primitive_type, ret_shape)
+
+
 def promoted_primitive_type(type1, type2):
     """
     Given a pair of tensor or primitive types, find the smallest type that can store an instance
@@ -105,7 +160,7 @@ def effective_kernel(kernel_shape, dilations):
             f"kernel_shape ({len(kernel_shape)}) and dilations ({len(dilations)}) "
             f"must be the same length"
         )
-    return [(k - 1) * d + 1 for k, d in zip(kernel_shape, dilations)]
+    return tuple([(k - 1) * d + 1 for k, d in zip(kernel_shape, dilations)])
 
 
 def aggregated_pad(
@@ -137,7 +192,7 @@ def aggregated_pad(
 
 
     Returns:
-        A list of total (before + after) padding for each spatial dimension in kernel_shape.
+        A tuple of total (before + after) padding for each spatial dimension in kernel_shape.
     """
     num_spatial_dims = len(kernel_shape)
     if dilations is None:
@@ -164,19 +219,20 @@ def aggregated_pad(
                 )
             )
         effective_ks = effective_kernel(kernel_shape, dilations)
-        return [
-            int(max(0, s * math.ceil(float(i) / float(s)) - i + k - s))
-            if not is_symbolic(i) else get_new_symbol()
-            for i, k, s in zip(input_shape, effective_ks, strides)
-        ]
+        return tuple(
+            [
+                int(max(0, s * math.ceil(float(i) / float(s)) - i + k - s))
+                if not is_symbolic(i)
+                else get_new_symbol()
+                for i, k, s in zip(input_shape, effective_ks, strides)
+            ]
+        )
     if pad_type == "valid":
-        return [0] * num_spatial_dims
+        return tuple([0] * num_spatial_dims)
     if pad_type == "custom":
         if custom_pad is None or len(custom_pad) != 2 * num_spatial_dims:
             raise ValueError("Invalid custom_pad.")
-        return [
-            custom_pad[2 * d] + custom_pad[2 * d + 1] for d in range(num_spatial_dims)
-        ]
+        return tuple([custom_pad[2 * d] + custom_pad[2 * d + 1] for d in range(num_spatial_dims)])
     raise ValueError('Invalid padding pad_type "{}"'.format(pad_type))
 
 
@@ -218,7 +274,7 @@ def spatial_dimensions_out_shape(
     if dilations is None:
         dilations = [1] * num_spatial_dims
     if custom_pad is None:
-        custom_pad = [0] * num_spatial_dims * 2
+        custom_pad = np.array([0] * num_spatial_dims * 2)
     if not (
         len(input_shape)
         == len(kernel_shape)
@@ -235,6 +291,22 @@ def spatial_dimensions_out_shape(
             "must all be the same length"
         )
 
+    effective_ks = effective_kernel(kernel_shape, dilations)
+    if isinstance(strides, np.ndarray):
+        strides = tuple(strides.tolist())
+    if isinstance(custom_pad, np.ndarray):
+        custom_pad = tuple(custom_pad.tolist())
+    cache_key = CONV_POOLING_TYPE_INFERENCE_CACHE.get_cache_key(
+        input_shape,
+        pad_type,
+        custom_pad,
+        strides,
+        effective_ks,
+        ceil_mode,
+    )
+    if cache_key in CONV_POOLING_TYPE_INFERENCE_CACHE:
+        return CONV_POOLING_TYPE_INFERENCE_CACHE[cache_key]
+
     pad = aggregated_pad(
         pad_type=pad_type,
         kernel_shape=kernel_shape,
@@ -243,7 +315,7 @@ def spatial_dimensions_out_shape(
         dilations=dilations,
         custom_pad=custom_pad,
     )
-    effective_ks = effective_kernel(kernel_shape, dilations)
+
     out_shape = []
     for r in range(num_spatial_dims):
         # only check if `input_shape` (spatial part of the input image) is symbolic, because:
@@ -264,10 +336,11 @@ def spatial_dimensions_out_shape(
             if out_dim <= 0:
                 raise ValueError(f"spatial dimension {r} has invalid output size {out_dim}")
             out_shape.append(out_dim)
+    CONV_POOLING_TYPE_INFERENCE_CACHE[cache_key] = out_shape
     return out_shape
 
 
-def parse_einsum_equation(equation: str) -> Tuple[List]:
+def parse_einsum_equation(equation: str) -> Tuple[List[str]]:
     """
     Args
         equation : str
@@ -295,28 +368,26 @@ def parse_einsum_equation(equation: str) -> Tuple[List]:
     output_str = input_output_str[1]
 
     inputs = input_str.split(',')
-    assert len(inputs) == 2, "unsupported einsum equation {}".format(equation)
-    input1_str = inputs[0]
-    input2_str = inputs[1]
 
-    input1_vec = [-1 for i in range(len(input1_str))]
-    input2_vec = [-1 for i in range(len(input2_str))]
-    output_vec = [-1 for i in range(len(output_str))]
+    in_outs = inputs + [output_str]
     map_char_to_int = {}
 
-    def _update_vec(str, vec, map_char_to_int, index):
+    def _update_vec(str, map_char_to_int, index):
+        vec = []
         for i, s in enumerate(str):
             if s not in map_char_to_int:
                 map_char_to_int[s] = index
                 index += 1
-            vec[i] = map_char_to_int[s]
-        return index
+            vec.append(map_char_to_int[s])
+        return index, vec
 
-    index = _update_vec(input1_str, input1_vec, map_char_to_int, 0)
-    index = _update_vec(input2_str, input2_vec, map_char_to_int, index)
-    _update_vec(output_str, output_vec, map_char_to_int, index)
+    index = 0
+    in_outs_vec = []
+    for inout_str in in_outs:
+        index, vec = _update_vec(inout_str, map_char_to_int, index)
+        in_outs_vec.append(vec)
 
-    return input1_vec, input2_vec, output_vec
+    return tuple(in_outs_vec)
 
 def compute_gather(params, indices, axis, batch_dims):
     """
@@ -387,17 +458,85 @@ def promote_input_dtypes(input_vars):
     return input_vars
 
 
+def get_squeeze_axes(squeeze_mask, rank):
+    """
+    Utility function to get the squeeze_axes from squeeze_mask.
+    i.e., returns a list of indices ``i`` where ``squeeze_mask[i] == True``.
+    For instance, given ``squeeze_mask = [True, False, True]``,
+    this utility returns ``[0, 2]``
+    """
+    if squeeze_mask is None:
+        squeeze_mask = [False] * rank
+    squeeze_axes = []
+    for idx, mask in enumerate(squeeze_mask):
+        if mask:
+            squeeze_axes.append(idx)
+    return squeeze_axes
+
+def get_param_val(param):
+    """
+    Given a param, if it is not None, returns param.val, else returns None.
+    """
+    if param is None:
+        return None
+    return param.val
+
+def solve_slice_by_index_slice(x_shape, begin, end, stride, begin_mask, end_mask, squeeze_mask):
+    """
+    Utility function to solve the slices of tensor slicing
+    """
+    # set default values for parameters
+    rank = len(x_shape)
+    begin = [int(i) for i in list(begin[:])]
+    end = [int(i) for i in list(end[:])]
+    if stride is None:
+        stride = [1] * rank
+    if begin_mask is None:
+        begin_mask = [False] * rank
+    if end_mask is None:
+        end_mask = [False] * rank
+    if squeeze_mask is None:
+        squeeze_mask = [False] * rank
+
+    # compute slices
+    slices = []
+    for idx, mask in enumerate(begin_mask):
+        if mask:
+            begin[idx] = None
+    for idx, mask in enumerate(end_mask):
+        if mask:
+            end[idx] = None
+    for idx, mask in enumerate(squeeze_mask):
+        if mask:
+            end[idx] = None
+            stride[idx] = np.iinfo(
+                np.int32
+            ).max  # We slice out only 1 element by setting stride to INF
+    for idx in range(rank):
+        slices.append(slice(begin[idx], end[idx], stride[idx]))
+
+    return tuple(slices)
+
 def solve_slice_by_index_shape(x_shape, begin, end, stride, begin_mask, end_mask, squeeze_mask):
     """
     Helper function to solve the shape of tensor slicing.
     """
-    ret_shape = []
-
+    # set default values
+    rank = len(x_shape)
     if begin is None or len(begin) == 0:
-        begin = [None] * len(x_shape)
+        begin = [None] * rank
     if end is None or len(end) == 0:
-        end = [None] * len(x_shape)
+        end = [None] * rank
+    if stride is None:
+        stride = [1] * rank
+    if begin_mask is None:
+        begin_mask = [False] * rank
+    if end_mask is None:
+        end_mask = [False] * rank
+    if squeeze_mask is None:
+        squeeze_mask = [False] * rank
 
+    # basic validation for tensor shape
     if len(begin) != len(x_shape):
         raise TypeError(
             "slice_by_index op: size of 'begin', {}, is not equal to the rank of input, which is {}".format(
@@ -412,6 +551,7 @@ def solve_slice_by_index_shape(x_shape, begin, end, stride, begin_mask, end_mask
         )
 
     # solve for shape inference
+    ret_shape = []
     for idx in range(len(x_shape)):
         # skip if we want to squeeze the dimension
         if squeeze_mask[idx]:
@@ -434,6 +574,13 @@ def solve_slice_by_index_shape(x_shape, begin, end, stride, begin_mask, end_mask
         """
         We first deal with those cases, where the output size is a deterministic number, even if the input dimension
         is unknown (i.e. symbolic)
+        - No begin_mask and no end_mask:
+          - begin == end. output shape = 0.
+          - begin == end - 1, stride > 0. output shape = 1
+          - begin == end + 1, stride < 0. output shape = 1
+        - begin_mask is false and end_mask is true:
+          - begin == -1, stride > 0. output shape = 1
+          - begin == 0, stride < 0. output shape = 1
         """
         if (
             not begin_mask[idx]
@@ -441,9 +588,7 @@ def solve_slice_by_index_shape(x_shape, begin, end, stride, begin_mask, end_mask
             and begin[idx] is not None
             and end[idx] is not None
         ):
-            # in this case the slice is from "begin" to "end", where both these boundary points are known
-            # we can find the size of the slice in this case, unless one of them is positive and other is negative
-            # as in that case, we would need to know the size of the full input dimension
+            out_shape = None
             if begin[idx] >= 0 and end[idx] >= 0 and stride[idx] > 0:
                 if end[idx] < begin[idx]:
                     raise ValueError(
@@ -452,12 +597,10 @@ def solve_slice_by_index_shape(x_shape, begin, end, stride, begin_mask, end_mask
                             idx, begin[idx], end[idx], stride[idx]
                         )
                     )
-                ret_shape.append(
-                    np.arange(end[idx] - begin[idx])[
-                        slice(0, end[idx] - begin[idx], stride[idx])
-                    ].size
-                )
-                continue
+                out_shape = np.arange(end[idx] - begin[idx])[
+                    slice(0, end[idx] - begin[idx], stride[idx])
+                ].size
+
             if begin[idx] < 0 and end[idx] < 0 and stride[idx] < 0:
                 if begin[idx] < end[idx]:
                     raise ValueError(
@@ -466,41 +609,17 @@ def solve_slice_by_index_shape(x_shape, begin, end, stride, begin_mask, end_mask
                             idx, begin[idx], end[idx], stride[idx]
                         )
                     )
-                ret_shape.append(
-                    np.arange(begin[idx] - end[idx])[
-                        slice(-1, end[idx] - begin[idx] - 1, stride[idx])
-                    ].size
-                )
-                continue
+                out_shape = np.arange(begin[idx] - end[idx])[
+                    slice(-1, end[idx] - begin[idx] - 1, stride[idx])
+                ].size
 
-        if begin_mask[idx] and not end_mask[idx] and end[idx] is not None:
-            # in this case we know that the slice is [0, end] or [-1, end], depending on the sign of stride,
-            #  and the value of end is known
-            if end[idx] > 0 and stride[idx] > 0:
-                ret_shape.append(
-                    np.arange(end[idx])[slice(None, end[idx], stride[idx])].size
-                )
-                continue
-            if end[idx] < 0 and stride[idx] < 0:
-                ret_shape.append(
-                    np.arange(abs(end[idx]))[slice(None, end[idx], stride[idx])].size
-                )
+            if out_shape in (0, 1):
+                ret_shape.append(out_shape)
                 continue
 
         if not begin_mask[idx] and end_mask[idx] and begin[idx] is not None:
-            # in this case we know the value of begin, and since end_mask is True, we know that the slice
-            # is till the right most edge
-            if begin[idx] > 0 and stride[idx] < 0:
-                ret_shape.append(
-                    np.arange(begin[idx] + 1)[slice(begin[idx], None, stride[idx])].size
-                )
-                continue
-            if begin[idx] < 0 and stride[idx] > 0:
-                ret_shape.append(
-                    np.arange(abs(begin[idx]))[
-                        slice(begin[idx], None, stride[idx])
-                    ].size
-                )
+            if (begin[idx] == 0 and stride[idx] < 0) or (begin[idx] == -1 and stride[idx] > 0):
+                ret_shape.append(1)
                 continue
 
         # for symbolic case
@@ -546,3 +665,90 @@ def solve_slice_by_index_shape(x_shape, begin, end, stride, begin_mask, end_mask
         ret_shape.append(max(0, num))
 
     return ret_shape
+
+
+def pack_elements_into_bits(elements: np.ndarray, nbits: int) -> np.ndarray:
+    """
+    Pack elements into nbits representation, by starting with the least significant bit (LSB) and
+    moving upward to the most significant bit (MSB).
+
+    Returns packed elements as np.uint8.
+    """
+    if not np.issubdtype(elements.dtype, np.integer):
+        raise ValueError(f"Only support packing integers elements, but got {elements.dtype}")
+
+    # Adjust allowed value range based on if the input is signed or unsigned.
+    if np.issubdtype(elements.dtype, np.signedinteger):
+        max_val = 2 ** (nbits - 1) - 1
+        min_val = -max_val - 1
+    else:
+        max_val = 2**nbits - 1
+        min_val = 0
+    if np.max(elements) > max_val:
+        raise ValueError(
+            f"To pack elements into {nbits}-bit, the max value is {max_val}, but got {np.max(elements)}"
+        )
+    if np.min(elements) < min_val:
+        raise ValueError(
+            f"To pack elements into {nbits}-bit, the min value is {min_val}, but got {np.min(elements)}"
+        )
+
+    # As np.unpackbits only supports uint8, convert to uint8 first.
+    # Notice that it will not lose information, because the bits are unchanged when converting int8
+    # to uint8. For example, the signed int -6 has bit representation '11111010', and when we unpackbits
+    # we get [0, 1, 0, 1, 1, 1, 1, 1], where only first 4 elements are needed for 4-bit representation.
+    elements = elements.astype(np.uint8)
+    bitarray = np.unpackbits(elements.reshape(-1, 1), bitorder="little", axis=-1)[:, :nbits]
+    return np.packbits(bitarray.flatten(), bitorder="little")
+
+
+def restore_elements_from_packed_bits(
+    packed_values: np.ndarray, nbits: int, element_num: int, are_packed_values_signed: bool = False
+) -> np.ndarray:
+    """
+    Restore elements from packed bits. Requires values that are packed by starting with the
+    least significant bit (LSB) and moving upward to the most significant bit (MSB), which is the
+    method used in `pack_elements_into_bits`.
+
+    are_packed_values_signed: Indicates if the packed_values were packed from signed integers. If
+        True, the n-bit number unpacked from packed_values will be interpreted as signed integers,
+        and the returned ndarray will have dtype np.int8. Otherwise, np.uint8 will be used.
+    """
+    if len(packed_values.shape) != 1:
+        raise NotImplementedError(
+            f"Only support 1-rank packed_values. But got {len(packed_values.shape)}"
+        )
+
+    if packed_values.dtype == np.int8:
+        # As np.unpackbits only supports uint8, need to convert first.
+        packed_values = packed_values.astype(np.uint8)
+    elif packed_values.dtype != np.uint8:
+        raise NotImplementedError(
+            f"Only support int8 or uint8 packed_values, but got {packed_values.dtype}"
+        )
+
+    bitarray = np.unpackbits(packed_values, bitorder="little")
+    pad_required = bitarray.size % nbits != 0
+    if pad_required:
+        bitarray = np.concatenate([bitarray, np.zeros(nbits - bitarray.size % nbits)]).astype(
+            bitarray.dtype
+        )
+        if bitarray.size % nbits != 0:
+            raise ValueError(
+                f"The length of bitarray ({bitarray.size}) should be divisible by "
+                f"nbits ({nbits})."
+            )
+    bitarray = bitarray.reshape(-1, nbits)[:element_num, :]
+    # The np.packbits doesn't work well for signed int if we feed `bitarray` to it directly.
+    # For example, the original signed int is -6, which is packed as 1010 for 4-bit representation,
+    # and here `bitarray` is [[0, 1, 0, 1]], where the value will be interpreted as 10 (b'1010')
+    # by np.packbits.
+    # To make np.packbits work correctly, we need to repeat the sign bit. For example, 1010 will
+    # become 11111010, where np.packbits can correctly handle and after converting to int8 it's -6.
+    if are_packed_values_signed:
+        # Repeat the sign bit to make uint8 to int8 works.
+        bitarray = np.repeat(bitarray, [1] * (nbits - 1) + [8 - nbits + 1], axis=1)
+    restored_elements = np.packbits(bitarray, bitorder="little", axis=-1).reshape(-1)
+    if are_packed_values_signed:
+        restored_elements = restored_elements.astype(np.int8)
+    return restored_elements
