@@ -8,14 +8,16 @@ import math
 import numpy as np
 
 from coremltools.converters.mil.mil import types
-from coremltools.converters.mil.mil.input_type import (DefaultInputs,
-                                                       InputSpec,
-                                                       TensorInputType)
-from coremltools.converters.mil.mil.operation import (SYMBOL, VALUE, Operation,
-                                                      precondition)
+from coremltools.converters.mil.mil.input_type import DefaultInputs, InputSpec, TensorInputType
+from coremltools.converters.mil.mil.operation import SYMBOL, VALUE, Operation, precondition
 from coremltools.converters.mil.mil.ops.defs._op_reqs import register_op
 from coremltools.converters.mil.mil.types import nptype_from_builtin
 from coremltools.converters.mil.mil.types.symbolic import is_symbolic
+from coremltools.converters.mil.mil.types.type_mapping import (
+    builtin_to_string,
+    string_to_builtin,
+    string_to_nptype,
+)
 
 
 def _maintain_shape(x, y):
@@ -489,7 +491,8 @@ class log(Operation):
 
     @precondition(allow=VALUE)
     def value_inference(self):
-        return np.log(self.x.val + self.epsilon.val)
+        result = np.log(self.x.val + self.epsilon.val)
+        return _maintain_shape(self.x.val, result)
 
 
 @register_op
@@ -602,7 +605,7 @@ class sign(elementwise_unary_with_int):
     """
     Return the sign value of the input ``x``, element-wise.
 
-    All elements in the output will be either ``-1``. or ``1``.
+    All elements in the output will be either ``-1`` or ``1``, or zero if the input ``x`` is zero.
 
     Parameters
     ----------
@@ -822,7 +825,7 @@ class cast(Operation):
     ----------
     x: tensor<[\*d], T> (Required)
     dtype: const str (Required)
-        * Can be one of the following types: ``int32``, ``int64``, ``fp32``, ``fp64``.
+        * Can be one of the following types: ``int32``, ``fp16``, ``fp32``, ``bool``.
 
     Returns
     -------
@@ -831,7 +834,7 @@ class cast(Operation):
 
     Attributes
     ----------
-    T: i32, i64, fp16, fp32, fp64, bool.
+    T: i32, fp16, fp32, bool.
     """
 
     input_spec = InputSpec(
@@ -840,59 +843,54 @@ class cast(Operation):
     )
 
     type_domains = {
-        "T": (types.fp16, types.fp32, types.fp64, types.int32, types.int64, types.bool),
+        "T": (types.fp16, types.fp32, types.int32, types.bool),
     }
 
-    def type_inference(self):
-        type_map = {
-            "int32": types.int32,
-            "int64": types.int32,
-            "fp16": types.fp16,
-            "fp32": types.fp32,
-            "fp64": types.fp32,
-            "bool": types.bool,
-        }
+    @classmethod
+    def supported_dtypes(cls):
+        return [builtin_to_string(v) for v in cls.type_domains["T"]]
 
-        if self.dtype.val not in type_map.keys():
+    def type_inference(self):
+        if self.dtype.val not in self.supported_dtypes():
             raise NotImplementedError(
                 "Parameter dtype of the cast operation can be one of the {}. "
-                "Provided {}".format(type_map.keys(), self.dtype.val)
+                "Provided {}".format(self.supported_dtypes(), self.dtype.val)
             )
 
         if not types.is_tensor(self.x.sym_type):
-            return type_map[self.dtype.val]
+            return string_to_builtin(self.dtype.val)
 
         ret_shape = self.x.shape
-        return types.tensor(type_map[self.dtype.val], ret_shape)
+        return types.tensor(string_to_builtin(self.dtype.val), ret_shape)
 
     @precondition(allow=VALUE | SYMBOL)
     def value_inference(self):
         return self.get_cast_value(self.x, self.dtype.val)
 
-    @staticmethod
-    def get_cast_value(input_var, dtype_val):
-        type_map = {
-            "int32": np.int32,
-            "int64": np.int32,
-            "fp16": np.float16,
-            "fp32": np.float32,
-            "fp64": np.float32,
-            "bool": bool,
-        }
-
-        if dtype_val not in type_map.keys():
+    @classmethod
+    def get_cast_value(cls, input_var, dtype_val):
+        if dtype_val not in cls.supported_dtypes():
             raise NotImplementedError(
                 "Parameter dtype of the cast operation can be one of the {}. "
-                "Provided {}".format(type_map.keys(), dtype_val)
+                "Provided {}".format(cls.supported_dtypes(), dtype_val)
             )
 
         if input_var.val is None:
-            if input_var.sym_val is not None and not is_symbolic(input_var.sym_val) and len(input_var.sym_val.shape) == 1:
-                result = [np.array(val).astype(dtype=type_map[dtype_val]).item() if not is_symbolic(val) else val for val in input_var.sym_val]
+            if (
+                input_var.sym_val is not None
+                and not is_symbolic(input_var.sym_val)
+                and len(input_var.sym_val.shape) == 1
+            ):
+                result = [
+                    np.array(val).astype(dtype=string_to_nptype(dtype_val)).item()
+                    if not is_symbolic(val)
+                    else val
+                    for val in input_var.sym_val
+                ]
                 return np.array(result)
             return None
 
         if not types.is_tensor(input_var.sym_type):
-            return input_var.val.astype(dtype=type_map[dtype_val])
+            return input_var.val.astype(dtype=string_to_nptype(dtype_val))
         else:
-            return np.array(input_var.val).astype(dtype=type_map[dtype_val])
+            return np.array(input_var.val).astype(dtype=string_to_nptype(dtype_val))
