@@ -39,7 +39,11 @@ from coremltools.converters.mil.mil.types.symbolic import (
 class band_part(Operation):
     """
     Returns a tensor setting everything outside a center band to zeros for the innermost
-    matrix. Special cases:
+    matrix. That is,
+    band(m, n) = (lower < 0 || (m-n) <= lower) && (upper < 0 || (n-m) <= upper)
+    output[i, j, k, ..., m, n] = band(m, n) * input[i, j, k, ..., m, n]
+
+    Special cases:
 
     - ``band_part(x, 0, -1)`` returns upper triangular part.
     - ``band_part(x, -1, 0)`` returns lower triangular part.
@@ -86,6 +90,19 @@ class band_part(Operation):
     def type_inference(self):
         return self.x.sym_type
 
+    @precondition(allow=VALUE)
+    def value_inference(self):
+        M, N = self.x.val.shape[-2:]
+        band = np.zeros((M, N), dtype=types.nptype_from_builtin(self.x.dtype))
+        num_lower = self.lower.val
+        num_upper = self.upper.val
+        for m in range(M):
+            for n in range(N):
+                band[m, n] = (num_lower < 0 or (m - n) <= num_lower) and (
+                    num_upper < 0 or (n - m) <= num_upper
+                )
+        return np.multiply(band, self.x.val)
+
 
 @register_op
 class cumsum(Operation):
@@ -445,6 +462,9 @@ class pad(Operation):
         * If mode is "replicate" then ``pad[2*i]`` and ``pad[2*i+1]`` can be
           at most ``D[i]``.
 
+        * If pad is not a constant, it must be a vector of length ``2 * rank(x)``,
+          that is, ``N == D_in``.
+
     mode: const<str> (Optional)
         * Defaults to ``constant``.
         * Must be one of the following values:
@@ -487,19 +507,27 @@ class pad(Operation):
         pad = self.pad
         if len(pad.shape) != 1:
             raise ValueError("Pad should be a 1D tensor!")
-        if self.mode and not self.mode.val in {'constant', 'reflect', 'replicate'}:
+        if self.mode and self.mode.val not in {"constant", "reflect", "replicate"}:
             raise ValueError("Pad mode should be one of {'constant', 'reflect', 'replicate'}")
+        if pad.val is None and pad.shape[0] != self.x.rank * 2:
+            raise ValueError(
+                f"Non-constant 'pad' must have shape ({2*self.x.rank},). Got {pad.shape}"
+            )
 
-        if pad.val is None:
+        if pad.sym_val is None:
             for i in range(self.pad.shape[0] // 2):
                 ret_shape[-self.pad.shape[0] // 2 + i] = get_new_symbol()
         else:
-            pad = pad.val
+            pad = pad.sym_val
             pad = pad.copy()
 
             if len(pad) % 2 != 0:
                 raise ValueError("Number of elements in the argument Pad must be divisible by 2.")
 
+            for i in range(len(pad)):
+                if not is_symbolic(pad[i]) and pad[i] < 0:
+                    raise ValueError(f"pad must be non-negative integer, got {pad[i]} at index {i}")
+
             pad = pad.reshape(-1, 2)
 
             if pad.shape[0] > len(ret_shape):
@@ -1267,6 +1295,9 @@ class stack(Operation):
         axis = self.axis.val
         if axis < 0:
             axis += (self.values[0].rank + 1)
+        rank = self.values[0].rank
+        if axis > rank:
+            raise ValueError(f"axis must in range [{-rank}, {rank}). Got {axis}")
         ret_shape = list(t_shape)
         ret_shape.insert(axis, num_tensors)
         return types.tensor(self.values[0].dtype, ret_shape)
