@@ -3,24 +3,34 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
+from typing import List
+
 import numpy as np
 import sympy as sm
 
 from coremltools import _logger as logger
-from coremltools.converters.mil.mil import (Operation, get_new_symbol,
-                                            get_new_variadic_symbol,
-                                            precondition, types)
-from coremltools.converters.mil.mil.input_type import (DefaultInputs,
-                                                       InputSpec,
-                                                       TensorInputType)
+from coremltools.converters.mil.mil import (
+    Operation,
+    get_new_symbol,
+    get_new_variadic_symbol,
+    precondition,
+    types,
+)
+from coremltools.converters.mil.mil.input_type import DefaultInputs, InputSpec, TensorInputType
 from coremltools.converters.mil.mil.operation import SYMBOL, VALUE
 from coremltools.converters.mil.mil.ops.defs._op_reqs import register_op
-from coremltools.converters.mil.mil.ops.defs._utils import \
-    solve_slice_by_index_shape
-from coremltools.converters.mil.mil.types.symbolic import (any_symbolic,
-                                                           any_variadic,
-                                                           is_symbolic,
-                                                           isscalar)
+from coremltools.converters.mil.mil.ops.defs._utils import (
+    get_param_val,
+    get_squeeze_axes,
+    solve_slice_by_index_shape,
+    solve_slice_by_index_slice,
+)
+from coremltools.converters.mil.mil.types.symbolic import (
+    any_symbolic,
+    any_variadic,
+    is_symbolic,
+    isscalar,
+)
 
 
 @register_op
@@ -214,9 +224,63 @@ class reshape(Operation):
         return val
 
     def _get_type_val(self):
-        x_type = self.x.dtype
-        x_shape = self.x.shape
-        x_vol = np.prod(x_shape)
+        count_neg_one = np.count_nonzero(self.shape.sym_val == -1)
+        if count_neg_one > 1:
+            raise ValueError(
+                f"Reshape op supports only one dimension to be -1, "
+                f"but got {count_neg_one} dimensions be -1."
+            )
+
+        if not any_symbolic(self.x.shape) and self.shape.val is not None:
+            ret_shape = self._infer_shape_static()
+        else:
+            ret_shape = self._infer_shape_dynamic()
+
+        ret_val = None
+        if self.x.sym_val is not None and all(
+            isscalar(a) and not is_symbolic(a) for a in ret_shape
+        ):
+            ret_val = reshape_with_symbol(self.x.sym_val, ret_shape)
+        return types.tensor(self.x.dtype, tuple(ret_shape)), ret_val
+
+    @staticmethod
+    def replace_zeros_in_shape(from_shape: List[int], to_shape: List[int]) -> List[int]:
+        """Replaces 0s in `to_shape` by the corresponding dims in `from_shape`."""
+        if to_shape.count(0):
+            if len(from_shape) != len(to_shape):
+                raise ValueError(
+                    f"When there is 0 in shape, the rank of x ({len(from_shape)}) "
+                    f"must equal to the target shape len ({len(to_shape)})."
+                )
+            to_shape = [s if s != 0 else from_shape[dim] for dim, s in enumerate(to_shape)]
+        return to_shape
+
+    @staticmethod
+    def replace_neg_one_in_shape(from_shape: List[int], to_shape: List[int]) -> List[int]:
+        """Replaces -1 in `to_shape` by the corresponding dims in `from_shape`."""
+        if to_shape.count(-1):
+            neg_one_idx = to_shape.index(-1)
+            total_element_num = np.prod(from_shape)
+            remain_element_num = np.prod(
+                [dim for idx, dim in enumerate(to_shape) if idx != neg_one_idx]
+            )
+            infer_dim = total_element_num // remain_element_num
+            to_shape[neg_one_idx] = infer_dim
+        return to_shape
+
+    def _infer_shape_static(self):
+        from_shape = list(self.x.shape)
+        to_shape = list(self.shape.val)
+        to_shape = self.replace_zeros_in_shape(from_shape, to_shape)
+        to_shape = self.replace_neg_one_in_shape(from_shape, to_shape)
+        if np.prod(from_shape) != np.prod(to_shape):
+            raise ValueError(
+                f"Invalid target shape in `reshape` op ({from_shape} to {list(self.shape.val)})."
+            )
+        return to_shape
+
+    def _infer_shape_dynamic(self):
+        x_vol = np.prod(self.x.shape)
         # shape is const, and thus sym_val is not None
         sym_shape = self.shape.sym_val
         sym_shape = [get_new_symbol() if d == -1 else d for d in sym_shape]
@@ -224,10 +288,7 @@ class reshape(Operation):
             ret_shape = reshape.enforce_volumetric_constraint(x_vol, sym_shape)
         except:
             ret_shape = sym_shape
-        ret_val = None
-        if self.x.val is not None and all(isscalar(a) and not is_symbolic(a) for a in ret_shape):
-            ret_val = reshape_with_symbol(self.x.val, ret_shape)
-        return types.tensor(x_type, tuple(ret_shape)), ret_val
+        return ret_shape
 
     @staticmethod
     def enforce_volumetric_constraint(left_volume, inshape):
@@ -240,13 +301,6 @@ class reshape(Operation):
 
         # Handling when reshape is given 0 instead of actual input
         # input tensor shape: [4, 3, 2], reshape:[0, -1], output tensor shape: [4, 6]
-        if shape.count(-1) > 1:
-            raise ValueError(
-                "Reshape op supports only one dimension to be -1. Given {}".format(
-                    shape.count(-1)
-                )
-            )
-
         infer_dim_index = shape.index(-1) if -1 in shape else None
         right_volume = 1
         for i in shape:
@@ -465,23 +519,16 @@ class slice_by_index(Operation):
             )
 
     def type_inference(self):
-
-        # get tensor and set default value
-        begin = self.begin.val
-        end = self.end.val
-        x_rank = self.x.rank
-        stride = self.stride.val if self.stride is not None else [1] * x_rank
-        begin_mask = (
-            self.begin_mask.val if self.begin_mask is not None else [False] * x_rank
-        )
-        end_mask = self.end_mask.val if self.end_mask is not None else [False] * x_rank
-        squeeze_mask = (
-            self.squeeze_mask.val if self.squeeze_mask is not None else [False] * x_rank
-        )
-
         # solve shape
-        x_shape = self.x.shape
-        ret_shape = solve_slice_by_index_shape(x_shape, begin, end, stride, begin_mask, end_mask, squeeze_mask)
+        ret_shape = solve_slice_by_index_shape(
+            self.x.shape,
+            self.begin.val,
+            self.end.val,
+            get_param_val(self.stride),
+            get_param_val(self.begin_mask),
+            get_param_val(self.end_mask),
+            get_param_val(self.squeeze_mask),
+        )
 
         if len(ret_shape) == 0:
             # Scalar case.
@@ -492,41 +539,21 @@ class slice_by_index(Operation):
     def value_inference(self):
         if self.x.sym_val is None or self.begin.val is None or self.end.val is None:
             return None
-        begin = [int(i) for i in list(self.begin.val[:])]
-        end = [int(i) for i in list(self.end.val[:])]
-        stride = [1] * self.x.rank if self.stride is None else self.stride.val
-        begin_mask = (
-            [False] * self.x.rank if self.begin_mask is None else self.begin_mask.val
-        )
-        end_mask = [False] * self.x.rank if self.end_mask is None else self.end_mask.val
-        squeeze_mask = (
-            [False] * self.x.rank
-            if self.squeeze_mask is None
-            else self.squeeze_mask.val
-        )
 
-        slices = []
-        for idx, mask in enumerate(begin_mask):
-            if mask:
-                begin[idx] = None
-        for idx, mask in enumerate(end_mask):
-            if mask:
-                end[idx] = None
-        squeeze_axes = []
-        for idx, mask in enumerate(squeeze_mask):
-            if mask:
-                end[idx] = None
-                stride[
-                    idx
-                ] = 2147483647  # We slice out only 1 element by setting stride to INF
-                squeeze_axes.append(idx)
-        for idx in range(self.x.rank):
-            slices.append(slice(begin[idx], end[idx], stride[idx]))
-
-        slices = tuple(slices)
+        # solve the data slices and slice tensor
+        slices = solve_slice_by_index_slice(
+            self.x.shape,
+            self.begin.val,
+            self.end.val,
+            get_param_val(self.stride),
+            get_param_val(self.begin_mask),
+            get_param_val(self.end_mask),
+            get_param_val(self.squeeze_mask),
+        )
         res = self.x.sym_val[slices]
 
-        # remove squeezed axes
+        # remove squeeze_axes
+        squeeze_axes = get_squeeze_axes(get_param_val(self.squeeze_mask), self.x.rank)
         if len(squeeze_axes) > 0:
             if len(squeeze_axes) == len(res.shape):
                 if len(res) == 0:
