@@ -56,7 +56,8 @@ class SSAOpRegistry:
         target.iOS13,
         target.iOS14,
         target.iOS15,
-        target.iOS16
+        target.iOS16,
+        target.iOS17,
     )
     core_ops = defaultdict(dict)
     dialect_ops = {}
@@ -97,7 +98,6 @@ class SSAOpRegistry:
         """
         def class_wrapper(op_cls):
             op_type = op_cls.__name__
-            op_cls.__name__ = op_type
 
             # debug message
             op_msg = "op"
@@ -116,10 +116,10 @@ class SSAOpRegistry:
                 # Check that op_type is prefixed with namespace
                 if op_type[: len(namespace)] != namespace:
                     msg = (
-                        "Dialect pp type {} registered under {} namespace must "
-                        + "prefix with {}"
+                        "Dialect op type {} registered under {} namespace must " + "prefix with {}"
                     )
                     raise ValueError(msg.format(op_type, namespace, namespace))
+                op_cls._dialect_namespace = namespace
             else:
                 op_reg = SSAOpRegistry.core_ops
 
