@@ -7,6 +7,7 @@
 from coremltools.converters.mil.backend.mil.passes import (
     adjust_io_to_supported_types,
     fuse_activation_silu,
+    fuse_pow2_sqrt,
     insert_image_preprocessing_op,
     sanitize_name_strings,
 )
@@ -37,6 +38,7 @@ from coremltools.converters.mil.mil.passes.defs import (
     optimize_elementwise_binary,
     optimize_linear,
     optimize_normalization,
+    optimize_quantization,
     optimize_repeat_ops,
     optimize_tensor_operation,
     preprocess,
