@@ -7,9 +7,15 @@ from typing import List
 
 import numpy as np
 
-from coremltools.converters.mil.mil import Block, Operation, Var
+from coremltools.converters.mil.mil import Block, Operation
 from coremltools.converters.mil.mil.passes.graph_pass import AbstractGraphPass
 
+class classproperty(property):
+    """
+    A decorator class that allow us to have a class-level property
+    """
+    def __get__(self, owner, cls):
+        return self.fget(cls)
 
 def block_context_manager(func):
     """
@@ -134,55 +140,3 @@ def _check_var_scalar_value(x, val, tol=1e-3):
     if abs(x_val - val) < tol:
         return True
     return False
-
-def _are_ops_identical(op1, op2):
-    '''
-    Return True, if all inputs of op1 and op2 are identical.
-    non-constant inputs must refer to the same object, and constant inputs must have the same value
-    '''
-
-    def _are_values_identical(val1, val2):
-        np_arr1 = np.array(val1)
-        np_arr2 = np.array(val2)
-        return np.array_equal(np_arr1, np_arr2)
-
-    def _are_vars_identical(var1, var2):
-        if var1.val is None and var2.val is None:
-            if var1 != var2:
-                return False
-        elif var1.val is not None and var2.val is not None:
-            if var1.dtype != var2.dtype:
-                return False
-            if not _are_values_identical(var1.val, var2.val):
-                return False
-        else:
-            return False
-        return True
-
-    if op1 == op2:
-        return True
-    if op1.op_type != op2.op_type:
-        return False
-    if len(op1.inputs) != len(op2.inputs):
-        return False
-
-    for key, value1 in op1.inputs.items():
-        if key not in op2.inputs:
-            return False
-        value2 = op2.inputs[key]
-        if isinstance(value1, Var) and isinstance(value2, Var):
-            if not _are_vars_identical(value1, value2):
-                return False
-        elif isinstance(value1, (list, tuple)) and isinstance(value2, (list, tuple)):
-            if len(value1) != len(value2):
-                return False
-            else:
-                for i, v in enumerate(value1):
-                    if not _are_vars_identical(v, value2[i]):
-                        return False
-        else:
-            return False
-
-    assert len(op1.blocks) == 0, "this method does not handle ops that have blocks in it"
-    assert len(op2.blocks) == 0, "this method does not handle ops that have blocks in it"
-    return True
