@@ -31,7 +31,11 @@ from .model import (
     _QUANTIZATION_MODE_DEQUANTIZE,
     _METADATA_VERSION,
     _METADATA_SOURCE,
+    _METADATA_SOURCE_DIALECT,
 )
 
 from . import neural_network
 from . import ml_program
+
+from ._compiled_model import CompiledMLModel
+
