@@ -15,7 +15,7 @@ import shutil as _shutil
 import subprocess as _subprocess
 import sys as _sys
 import tempfile as _tempfile
-from typing import Optional as _Optional
+from typing import Optional as _Optional, Union as _Union
 import warnings as _warnings
 
 import numpy as _np
@@ -124,10 +124,10 @@ def save_spec(spec, filename, auto_set_specification_version=False, weights_dir=
         Protobuf representation of the model
 
     filename: str
-        File path  where the spec gets saved.
+        File path where the spec gets saved.
 
     auto_set_specification_version: bool
-        If true, will always try to set specification version automatically.
+        If True, will always try to set specification version automatically.
 
     weights_dir: str
         Path to the directory containing the weigths.bin file. This is required
@@ -1004,13 +1004,84 @@ def convert_double_to_float_multiarray_type(spec):
             convert_double_to_float_multiarray_type(model_spec)
 
 
-def make_pipeline(*models):
+def compile_model(model: _Union['_ct.models.MLModel', str, _Model_pb2.Model]) -> str:
+    """
+    Compiles a Core ML model.
+
+    Parameters
+    ----------
+    model: str, Model_pb2 or MLModel
+
+        str : Path to model to compile
+
+        Model_pb2 : Spec to model to compile
+
+        MLModel : Instantiated Core ML model to compile
+
+    Returns
+    -------
+
+    str : Path to compiled model directory
+
+    See Also
+    --------
+    coremltools.models.CompiledMLModel
+    """
+    # Check environment
+    if _macos_version() < (10, 13):
+        raise Exception("Compiling a Core ML models is only support on macOS 10.13 or higher.")
+    try:
+        from ..libcoremlpython import _MLModelProxy
+    except:
+        raise Exception("Unable to compile any Core ML models.")
+
+    # Check parameter
+    if not isinstance(model, (str, _Model_pb2.Model, _ct.models.MLModel)):
+        raise Exception("Compiling a Core ML models is only support on macOS 10.13 or higher.")
+
+    # Compile model
+    if isinstance(model, (_Model_pb2.Model, _ct.models.MLModel)):
+        if isinstance(model, _ct.models.MLModel):
+            spec = model.get_spec()
+        else:
+            spec = model
+
+        with _tempfile.TemporaryDirectory() as save_dir:
+            spec_file_path = save_dir + '/spec.mlmodel'
+            save_spec(spec, spec_file_path)
+            return _MLModelProxy.compileModel(spec_file_path)
+    else:
+        assert isinstance(model, str)
+        model = _os.path.expanduser(model)
+        return _MLModelProxy.compileModel(model)
+
+
+def make_pipeline(
+        *models: '_ct.models.MLModel',
+        compute_units: _Union[None, _ct.ComputeUnit] = None
+    ) -> '_ct.models.MLModel':
     """
     Makes a pipeline with the given models.
 
     Parameters
     ----------
-    *models - two or more instances of ct.models.MLModel
+    *models
+        Two or more instances of ``ct.models.MLModel``.
+
+    compute_units: ``None`` or ``coremltools.ComputeUnit``
+        The set of processing units that all models in the pipeline can use to make predictions.
+
+        If None, the ``compute_unit`` will be infered from the ``compute_units`` values of the models.
+        If all models do not have the same ``compute_units`` values, this parameter must be specified.
+
+        ``coremltools.ComputeUnit`` is an enum with four possible values:
+            - ``coremltools.ComputeUnit.ALL``: Use all compute units available, including the
+                neural engine.
+            - ``coremltools.ComputeUnit.CPU_ONLY``: Limit the model to only use the CPU.
+            - ``coremltools.ComputeUnit.CPU_AND_GPU``: Use both the CPU and GPU,
+                but not the neural engine.
+            - ``coremltools.ComputeUnit.CPU_AND_NE``: Use both the CPU and neural engine, but
+                not the GPU. Available only for macOS >= 13.0.
 
     Returns
     -------
@@ -1018,10 +1089,18 @@ def make_pipeline(*models):
 
     Examples
     --------
-    my_model1 = ct.models.MLModel('/tmp/m1.mlpackage')
-    my_model2 = ct.models.MLModel('/tmp/m2.mlmodel')
+    .. sourcecode:: python
+
+        my_model1 = ct.models.MLModel('/tmp/m1.mlpackage')
+        my_model2 = ct.models.MLModel('/tmp/m2.mlmodel')
+        
+        my_pipeline_model = ct.utils.make_pipeline(my_model1, my_model2)
+
+        y = my_pipeline_model.predict({'x': 12})
+
+        my_pipeline_model.save('/tmp/my_pipeline.mlpackage')
+        new_my_pipeline = ct.model.MLModel('/tmp/my_pipeline.mlpackage')
 
-    my_pipeline_model = ct.utils.make_pipeline(my_model1, my_model2)
     """
 
     def updateBlobFileName(proto_message, new_path):
@@ -1045,6 +1124,28 @@ def make_pipeline(*models):
 
 
     assert len(models) > 1
+    if compute_units is not None and not isinstance(compute_units, _ComputeUnit):
+        raise TypeError('"compute_units" parameter must be None or of type coremltools.ComputeUnit')
+
+    if compute_units is None:
+        all_compute_units_the_same = all(map(
+            lambda m: models[0].compute_unit is m.compute_unit,
+            models[1:]
+        ))
+        if not all_compute_units_the_same:
+            raise ValueError(
+                'Models have different compute_unit values. The "compute_units" parameter must be specified.'
+            )
+        compute_units = models[0].compute_unit
+
+    if (compute_units == _ComputeUnit.CPU_AND_NE
+          and _is_macos()
+          and _macos_version() < (13, 0)
+          ):
+        raise ValueError(
+            'coremltools.ComputeUnit.CPU_AND_NE is only available on macOS >= 13.0'
+        )
+
     input_specs = list(map(lambda m: m.get_spec(), models))
 
     pipeline_spec = _ct.proto.Model_pb2.Model()
@@ -1052,15 +1153,24 @@ def make_pipeline(*models):
         map(lambda spec: spec.specificationVersion, input_specs)
     )
 
-    # Set pipeline input
-    pipeline_spec.description.input.MergeFrom(
-        input_specs[0].description.input
-    )
+    # If a later model doesn't get an input from a previous model, it must be
+    # an input to the pipeline.
+    available_as_input = set()
+    for cur_spec in input_specs:
+        for cur_input in cur_spec.description.input:
+            if cur_input.name not in available_as_input:
+                pipeline_spec.description.input.add().MergeFrom(cur_input)
+                available_as_input.add(cur_input.name)
+        available_as_input.update([i.name for i in cur_spec.description.output])
 
-    # Set pipeline output
-    pipeline_spec.description.output.MergeFrom(
-        input_specs[-1].description.output
-    )
+    # If an output for a model is not used as input for a later model, assume it
+    # should be an output to the pipeline.
+    used_as_input = set()
+    for cur_spec in input_specs[::-1]:     # iterate overs specs in reverse
+        for cur_output in cur_spec.description.output:
+            if cur_output.name not in used_as_input:
+                pipeline_spec.description.output.add().MergeFrom(cur_output)
+        used_as_input.update([i.name for i in cur_spec.description.input])
 
     # Map input shapes to output shapes
     var_name_to_type = {}
@@ -1094,4 +1204,4 @@ def make_pipeline(*models):
             if _os.path.exists(weight_file_path):
                 _shutil.copyfile(weight_file_path, dst + f"/{i}-weight.bin")
 
-    return _ct.models.MLModel(pipeline_spec, weights_dir=dst)
+    return _ct.models.MLModel(pipeline_spec, compute_units=compute_units, weights_dir=dst)
