@@ -43,6 +43,7 @@ EXPECTED_MODULES = [
     "transform",
     "libmodelpackage",
     "libmilstoragepython",
+    "optimize",
 ]
 
 
@@ -56,6 +57,7 @@ class TestApiVisibilities:
 
     def test_utils(self):
         expected = [
+            "compile_model",
             "convert_double_to_float_multiarray_type",
             "evaluate_classifier",
             "evaluate_classifier_with_probabilities",
@@ -70,6 +72,7 @@ class TestApiVisibilities:
 
     def test_models(self):
         expected = [
+            "CompiledMLModel",
             "MLModel",
             "datatypes",
             "feature_vectorizer",
@@ -86,6 +89,7 @@ class TestApiVisibilities:
     def test_models_mlmodel(self):
         expected = [
             "author",
+            "get_compiled_model_path",
             "get_spec",
             "input_description",
             "license",
@@ -160,6 +164,30 @@ class TestApiVisibilities:
         ]
         _check_visible_modules(_get_visible_items(ct.converters), expected)
 
+    def test_optimize(self):
+        expected = [
+            "coreml",
+            "torch",
+        ]
+        _check_visible_modules(_get_visible_items(ct.optimize), expected)
+
+    def test_optimize_coreml(self):
+        expected = [
+            "OpLinearQuantizerConfig",
+            "OpMagnitudePrunerConfig",
+            "OpPalettizerConfig",
+            "OptimizationConfig",
+            "OpThresholdPrunerConfig",
+            "linear_quantize_weights",
+            "palettize_weights",
+            "prune_weights",
+            "decompress_weights",
+            "get_weights_metadata",
+            "CoreMLWeightMetaData",
+            "CoreMLOpMetaData",
+        ]
+        _check_visible_modules(_get_visible_items(ct.optimize.coreml), expected)
+
     def test_converters_libsvm(self):
         _check_visible_modules(_get_visible_items(ct.converters.libsvm), ["convert"])
 
