@@ -12,13 +12,16 @@ import pytest
 
 import coremltools
 from coremltools import ComputeUnit, utils
+from coremltools._deps import _HAS_TORCH
 from coremltools.converters.mil import Builder as mb
 from coremltools.libmodelpackage import ModelPackage
 from coremltools.models import MLModel
-from coremltools.models.utils import (_MLPACKAGE_AUTHOR_NAME,
-                                      _WEIGHTS_DIR_NAME)
+from coremltools.models.utils import _MLPACKAGE_AUTHOR_NAME, _WEIGHTS_DIR_NAME
 from coremltools.proto import Model_pb2
 
+if _HAS_TORCH:
+    import torch
+
 
 def _remove_path(path):
     if os.path.isdir(path):
@@ -26,6 +29,7 @@ def _remove_path(path):
     else:
         os.remove(path)
 
+
 class TestMLModel:
 
     def setup_class(self):
@@ -100,15 +104,44 @@ class TestMLModel:
         model.save(package.name)
         loaded_model = MLModel(package.name)
 
-        assert model.author == "Test author"
-        assert model.license == "Test license"
-        assert model.short_description == "Test model"
-        assert model.input_description["feature_1"] == "This is feature 1"
-        assert model.output_description["output"] == "This is output"
+        assert loaded_model.author == "Test author"
+        assert loaded_model.license == "Test license"
+        assert loaded_model.short_description == "Test model"
+        assert loaded_model.input_description["feature_1"] == "This is feature 1"
+        assert loaded_model.output_description["output"] == "This is output"
 
         # cleanup
         _remove_path(package.name)
 
+
+    @pytest.mark.skipif(not _HAS_TORCH, reason="requires torch")
+    def test_save_from_mlpackage(self):
+        class Model(torch.nn.Module):
+            def forward(self, x):
+                return x
+
+        example_input = torch.rand(1, 3, 50, 50)
+        traced_model = torch.jit.trace(Model().eval(), example_input)
+
+        model = coremltools.convert(
+            traced_model,
+            inputs=[coremltools.TensorType(shape=example_input.shape)],
+            convert_to="mlprogram",
+        )
+
+        author = "Bobby Joe!"
+        model.author = author
+
+        save_dir = tempfile.TemporaryDirectory(suffix=".mlpackage").name
+
+        model.save(save_dir)
+        loaded_model = MLModel(save_dir)
+
+        assert loaded_model.author == author
+
+        _remove_path(save_dir)
+
+
     def test_predict_api(self):
         model = MLModel(self.spec)
 
@@ -127,6 +160,7 @@ class TestMLModel:
 
                 preds = loaded_model.predict({"feature_1": 1.0, "feature_2": 1.0})
                 assert preds is not None
+                assert len(preds.keys()) == 1
                 assert preds["output"] == 3.1
                 assert loaded_model.compute_unit == compute_units
         else:
@@ -136,6 +170,23 @@ class TestMLModel:
         # cleanup
         _remove_path(package.name)
 
+
+    @pytest.mark.skipif(utils._macos_version() < (12, 0),
+                        reason="prediction available only on macOS12+")
+    def test_batch_predict(self):
+        model = MLModel(self.spec)
+        x = [ {"feature_1": 1.0, "feature_2": 1.0},
+              {"feature_1": 2.0, "feature_2": 2.0} ]
+
+        y = model.predict(x)
+
+        assert len(y) == 2
+        assert y[0]["output"] == 3.1
+        assert len(y[0].keys()) == 1
+        assert y[1]["output"] == 6.1
+        assert len(y[1].keys()) == 1
+
+
     def test_rename_input(self):
         utils.rename_feature(self.spec, "feature_1", "renamed_feature", rename_inputs=True)
         model = MLModel(self.spec)
@@ -214,15 +265,14 @@ class TestMLModel:
 
         _remove_path(package.name)
 
+    @pytest.mark.skipif(not _HAS_TORCH, reason="requires torch")
     def test_mil_as_package(self):
-        import torch
-
         num_tokens = 3
         embedding_size = 5
 
         class TestModule(torch.nn.Module):
             def __init__(self):
-                super(TestModule, self).__init__()
+                super().__init__()
                 self.embedding = torch.nn.Embedding(num_tokens, embedding_size)
 
             def forward(self, x):
@@ -292,7 +342,7 @@ class TestMLModel:
 
         class TestModule(torch.nn.Module):
             def __init__(self):
-                super(TestModule, self).__init__()
+                super().__init__()
                 self.embedding = torch.nn.Embedding(num_tokens, embedding_size)
 
             def forward(self, x):
