@@ -7,6 +7,7 @@ import itertools
 import unittest
 
 import numpy as np
+import pytest
 
 import coremltools.models.datatypes as datatypes
 from coremltools import ComputeUnit
@@ -164,6 +165,11 @@ class StressTest(CorrectnessTest):
         self.test_data_reorganize(cpu_only=True)
 
     def test_depthwise_conv(self, cpu_only=False):
+        if not cpu_only:
+            pytest.xfail(
+                "rdar://116060011: re-activate coremltools tests blocked by Core ML regressions"
+            )
+
         def get_coreml_model_depthwise(X, params, w):
             eval = True
             mlmodel = None
@@ -390,6 +396,9 @@ class StressTest(CorrectnessTest):
 
     @unittest.skipUnless(_macos_version() >= (10, 14), "Only supported on MacOS 10.14+")
     def test_crop_resize(self, cpu_only=False):
+        if _macos_version()[0] == 12:
+            pytest.xfail("rdar://110274216")
+
         def get_coreml_model_crop_resize(params):
             eval = True
             mlmodel = None
